/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <strings.h>

#include "memm.h"
#include "configcraft.h"


ConfigCraft::ConfigCraft(char *programtitle, int Num, char *Opts[], int link)
  {
    /* We have to look for the -M to get the filename */
    num = Num;
    opts = Opts;
    /* New link feature 9/7/00 If there's a /etc/NAME.cf use that */
    Memory t(strlen(programtitle)+10);
    strcpy (t, "");
    if (link)
      strcpy (t, "/etc/");
    strcat (t, programtitle);
    strcat (t, ".cf");
    filename = t;

    if (link) // if link is on, but there's no link file, revert to local directory
      { // 1/11/03
        FILE *f = fopen(filename, "r");
        if (f == NULL)
          {
            strcpy (filename, programtitle);
            strcat (filename, ".cf");
          }
        else
          fclose(f);
      }
    
    // check the params.
    cmdlinesize = 0;
    char *bmem = getmem(10);

    int lp = 1;
    while (lp < num)
      {
        if (memcmp (opts[lp], "-M", 2) == 0)
          {
            // see if it's attached or next param
            if (strlen(opts[lp]) > 2)
              filename = opts[lp]+2; // copy the filename to use
            else
              {
                lp++;
                if (lp < num)
                  filename = opts[lp]; // copy the next param, must be the name
              }
          } // otherwise, we just use default, of progname.cf
        // see if we want to add anything to config file. If so
        // do it before we read it in, so we get precedence.
        Memory en;
        if (memcmp (opts[lp], "-S", 2) == 0)
          {
            // see if it's attached or next param
            if (strlen(opts[lp]) > 2)
              en = opts[lp]+2; // copy the param to use
            else
              {
                lp++;
                if (lp < num)
                  en = opts[lp]; // copy the next param, must be the name
              }
            // add this to cmdlinestring
            bmem = reallocmem(bmem, cmdlinesize+strlen(en)+5);
            memcpy(bmem+cmdlinesize, en, strlen(en));
            cmdlinesize += strlen(en); // go to where to put the null
            *(bmem+cmdlinesize) = '\0';
            cmdlinesize++;
            *(bmem+cmdlinesize) = '\0'; // put trailing null in case this is the end
          }
        lp++;
      }
    cmdlineopts.steal(bmem); // cmdlinesize is set correctly
    refresh(); // go read it.
  }

ConfigCraft::~ConfigCraft(void)
  {
    // hmm, everything deallocates itself. nothing to do
  }
      
void ConfigCraft::refresh(void)
  {
    /* Read in the config file. turn it into a
       bunch of xxyzzy:data\0 pairs, trailed by \0\0
       Skip the lines starting with '#' or that don't have ':' 
       in them. */
    FILE *f;
    
    Memory empty("\0\0", 2);
    image.steal(empty.victim());

    f = fopen (filename, "r");
    if (f == NULL)
      return;

    // get the max size of this puppy
    long size = sizereset(f) + cmdlinesize + 10;
    
    Memory lin(size); // can't possibly lose.
    Memory holding(size);
    memcpy (holding, cmdlineopts, cmdlinesize+1); // copy double 0 in case we add nothing
    currentpos = holding + cmdlinesize;

    int ret = readline(lin, f);
    while (ret > 0)
      {
        trim (lin);
        if (*lin != '#') // skip comments
          {
            char *pos;
            if ((pos = strchr(lin, ':')) != NULL) // skip lines with no :
              {
                *pos = '\0';
                Memory left(lin);
                Memory right(pos+1);
                trim (left);
                trim (right);
                strcpy(currentpos, left);
                strcat(currentpos, ":");
                strcat(currentpos, right);
                currentpos += strlen(currentpos) + 1; // go after line ending \0
              }
          } // if not a comment
        ret = readline(lin, f);
      }
    if (currentpos == (char *)holding)
      currentpos++; // in the case of empty file
      
    *currentpos = '\0'; // put the trailing double \0
    Memory fin(holding, currentpos - (char *)holding + 2); // get just the size we use
    image.steal(fin.victim()); // we can't do = because it does a strcpy
    fclose (f);
  }
  
char *ConfigCraft::operator[](char *s)
  {
    currentpos = image; // start at top
    
    while (currentpos != NULL)
      {
        Memory t(currentpos);
        char *zot;
        if ((zot = strchr (t, ':')) != NULL)
          {   
            *zot = '\0';
            if (strcasecmp(t, s) == 0)
              return (strchr (currentpos, ':')+1); // skip the :
          }
        currentpos = nextline();
      }
    return ""; // not found, don't return NULL, somebody might deref
  }
      
char *ConfigCraft::nextline(void)
  {
    while (*currentpos != '\0')
      currentpos++;

    currentpos++; // skip the first ends;
    if (*currentpos != '\0')
      return currentpos;
    currentpos--; // so next time we stay at end
    return NULL;
  }

char *ConfigCraft::operator[](unsigned char *s)  
  {
    return (*this)[(char *)s];
  }

void ConfigCraft::adddefault(char *option, char *value)
  {
    /* Adding to the end makes it a default because we search 
       from top to bottom, if there's one already there, we'll use that. */
    Memory left(option);
    Memory right(value);
    trim (left);
    trim (right);
    
    Memory add(strlen(left) + strlen(right) + 5);
    strcpy (add, left);
    strcat (add, ":");
    strcat (add, right);
    
    /* Now add to the end of image */
    currentpos = image;
    while (nextline()); // find the end
    if (currentpos != image) // don't skip first byte if image is empty */
      currentpos++;
    
    Memory newimage(currentpos - (char *)image + strlen(add) + 2); // get just the size we use
    memcpy (newimage, image, currentpos - (char *)image + 1); // copy the old to new
    image.steal(newimage.victim());
    // go to the end of this new space
    currentpos = image;
    while (nextline()); // find the end
    if (currentpos != image) // don't skip first byte if image is empty */
      currentpos++;
    strcpy (currentpos, add);
    *(currentpos + strlen(add) + 1) = '\0'; // make sure to double \0\0    
  }

int ConfigCraft::getcmdlineparam(char letter, Memory &param)
  {
    /* This could be in memm, but we already have num and opts,
       so it's useful here too. 
       Look for the command line param -<letter> if there's
       something following it, return it in param.
       return TRUE if found, FALSE if not. */

    param = ""; // reset
    int lp = 1;
    while (lp < num)
      {
        char cmp[2];
        cmp[0] = '-'; cmp[1] = letter;
        if (memcmp (opts[lp], cmp, 2) == 0)
          {
            // see if it's attached or next param
            if (strlen(opts[lp]) > 2)
              param = opts[lp]+2; // skip the -<letter> take the param
            else
              {
                lp++;
                if (lp < num)
                  param = opts[lp]; // copy the next param, must be the option if any
              }
            return TRUE;
          }
        lp++;
      }
    return FALSE; // didn't find it.
  }
