/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef CONFIGCRAFT_H
#define CONFIGCRAFT_H

#include <stdio.h>
#include <strstream.h>
#include "memm.h"

/* Instructions:

   ConfigCraft(char *programtitle, int num, char *opts[], int link);
     programtitle is for picking the name of the config file.
     If programtitle is "test" then the configfile "test.cf" will be read.
     num and opts are argv and argc as passed to main. This is so the -M and -S
     can be interpreted.
     -M<filename> on the command line specifies the configfile to read out of.
     -S<name:value> overrides any name in the config file with this value
      
   void refresh(void);
     If for whatever reason you want to re-read the config file, do it with this.
     Any values passed on the command line will remain and override the config
     file.

   #define val atoi // for convienence.
   
   char *operator[](char *);
   char *operator[](unsigned char *);
     To read a value from the config file. Search is not case sensitive.
     
     ConfigCraft ar("test", argv, argc);
     
     cout << "server: " << ar["server"] << endl;
     cout << "port: " << val(ar["port"]) << endl;

   void adddefault(char *option, char *value);
     To add a default value to the confile file list, use this function.
     The search is done from the top of the list to the bottom. This function
     adds an item to the end of the list. So if there's nothing in the file, you'll
     get the first default option added. If there is something in the file,
     it takes precedence.

   void setfilename(char *name);
     Not quite sure why I put this here, but I must have had a reason. If you
     want to change the name of the config file after creating the object,
     pass it here, then you can refresh() and it will reread the file.

   int getcmdlineparam(char letter, Memory &param);
     If you want to check for a command line param in your application
     you can do it with this. Only accepts one letter to check for
     as in -dparam or -d param  It will return the param if any
     in param, and TRUE or FALSE if not found.
   
*/


/* My simple config file reader */

/* Works like this, refresh reads in the file line by line makes a nt string out of value:data.
   When somebody asks, we whip through the strings, we know how many there are, and
   return a pointer to the data part which always exists and is a NT string itself. */

class ConfigCraft
  {
    public:
      ConfigCraft(char *programtitle, int num, char *opts[], int link = FALSE);
      ~ConfigCraft(void);
      
      void refresh(void);
      char *operator[](char *);
      char *operator[](unsigned char *);
      void adddefault(char *option, char *value);
      void setfilename(char *name) { filename = name; }
      int getcmdlineparam(char letter, Memory &param);
          
      int getnum(void) { return num; }
      char **getopts(void) { return opts; }
          
    private:
      Memory image;
      Memory filename;
      Memory cmdlineopts;
      int cmdlinesize;
      
      char *nextline(void);
      char *currentpos;
      
      int num;
      char **opts;
  };

#endif

