/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "filelist.h"

#ifdef __XLC__
extern "C"
  {
    #include <sys/types.h>
    #include <sys/dir.h>
  }
#else
  #include <sys/dir.h>
#endif

#include <stdlib.h>
#include <strings.h>

Memory filelist::ext;
Memory filelist::prefix;

filelist::filelist(char *directory, char *extention, char *pfx, int sorttype)
  {
    // construct the list and read it
    ext = extention; // so the callback can check it
    prefix = pfx;
    files = NULL; // 1/7/03, how did I miss this? If call fails, this better not be junk

#ifdef __XLC__
    if (sorttype == filelist::SORTNAME)
      count = scandir (directory, &files, (int(*)(dirent*))sel, alphasort);
    else
      count = scandir (directory, &files, (int(*)(dirent*))sel, NULL);
#else
    if (sorttype == filelist::SORTNAME)
      count = scandir (directory, &files, (int(*)(const struct dirent *))sel, alphasort);
    else
      count = scandir (directory, &files, (int(*)(const struct dirent *))sel, NULL);
#endif
  }
  
filelist::~filelist(void)
  {
    int lp;   
    if (files == NULL)
      return;
    for (lp = 0; lp < count; lp++)
      free(files[lp]); 
    free (files);
  }

Memory &filelist::operator[](int num)
  {
    struct dirent *dp;
      
    if ((num >= count) || (num < 0))
      turner = "";
    else
      {
        dp = files[num];
        turner = dp->d_name;
      }
    return turner;
  }
    
int sel(const struct dirent *checkit)
  {
    // checkit versus extention. Let's see if they buy this.

    /* Hmm... I just noticed that ext and prefix are stored as
       Memory which means they'll never come back NULL. Too bad
       that would have been handy. Don't wanna break anything old though. */
    char *ext = filelist::getext();
    char *prefix = filelist::getprefix();
    
    int ret = TRUE;
      
    if (strlen(ext) > 0)
      {
        int lext = strlen(ext);
        char *name = (char *)checkit->d_name;
        int lname = strlen(name);
        if (lname < lext) // bounds check
          ret = FALSE; // if prefix is longer then it can't possibly match
        else
          if (strcasecmp(ext, name + (lname - lext)) != 0)
            ret = FALSE;
      }

    if (strlen(prefix) > 0)
      {
        int lprefix = strlen(prefix);
        Memory name((char *)checkit->d_name);
        int lname = strlen(name);
        if (lname < lprefix) // bounds check
          ret = FALSE; // if prefix is longer then it can't possibly match
        else
          { // only check the first chars in the prefix
            name[strlen(prefix)] = '\0';
            if (strcasecmp(prefix, name) != 0)
              ret = FALSE;
          }
      }
     
    return ret;
  }

