/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* My log stream */

#include <time.h>
#include <unistd.h>
#include "notecraft.h"
#include "memm.h"

NoteCraft::NoteCraft(char *programname, char *filename, int minlevel)
  { // can be 'cout'

    progname = programname;
    notelevel = minlevel;
    logfile = NULL;
    levelset = 0;
    freshline = TRUE;
    if (strcmp (filename, "cout") == 0)
      {
        logfile = (FILE *)this; // an interesting flag to denote cout
      }
    else
      {
        logfile = fopen (filename, "a");
      }
  }
  
NoteCraft::~NoteCraft(void)
  {
    if (logfile == NULL)
      return;
    if (logfile != (FILE *)this)
      fclose (logfile);
  }

NoteCraft &NoteCraft::operator()(int level) 
  { 
    if (logfile == NULL)
      return *this;

    levelset = level; 

    freshline = TRUE;
    return *this;
  }
      
void NoteCraft::sout (char *s)
  {
    /* Tack the time and pid on the front */
    
    if (logfile == NULL)
      return;
      
    if (levelset < notelevel)
      return;
    
    Memory ts(strlen(s) + 16 + strlen(progname) + 10 + 5 + 5);
    if (freshline != FALSE)
      {
        Memory ct(30);
        time_t t = time(NULL);
#ifdef _THREAD_SAFE
        ctime_r(&t, ct);
#else
        ct = ctime(&t);
#endif
        memcpy (ts, ct+4, 15);
        sprintf (ts+15, " %s [%d]:[%d] ", (char *)progname, getpid(), levelset);
        strcat (ts, s);
        freshline = FALSE;
      }
    else
      strcpy (ts, s);

    if (logfile == (FILE *)this)
      printf ("%s", (char *)ts);
    else
      fprintf (logfile, "%s", (char *)ts);  
  }

NoteCraft &NoteCraft::operator<<(char c)
  {
    if (logfile == NULL)
      return *this;
    char s[2];
    s[0] = c;
    s[1] = '\0';
    sout(s);
    return *this;
  }

NoteCraft &NoteCraft::operator<<(unsigned char c)
  {
    if (logfile == NULL)
      return *this;
    char s[2];
    s[0] = (char)c;
    s[1] = '\0';
    sout(s);
    return *this;
  }
  
NoteCraft &NoteCraft::operator<<(char *s)
  {
    if (logfile == NULL)
      return *this;
    sout(s);
    return *this;
  }
  
NoteCraft &NoteCraft::operator<<(unsigned char *s)
  {
    if (logfile == NULL)
      return *this;
    sout ((char *)s);
    return *this;
  }
  
NoteCraft &NoteCraft::operator<<(int n)
  {
    if (logfile == NULL)
      return *this;
    Memory s(20);
    sprintf (s, "%d", n);
    sout (s);
    return *this;
  }

NoteCraft &NoteCraft::operator<<(unsigned int n)
  {
    if (logfile == NULL)
      return *this;
    Memory s(20);
    sprintf (s, "%u", n);
    sout (s);
    return *this;
  }

NoteCraft &NoteCraft::operator<<(long n)
  {
    if (logfile == NULL)
      return *this;
    Memory s(20);
    sprintf (s, "%lu", n);
    sout (s);
    return *this;
  }

NoteCraft &NoteCraft::operator<<(unsigned long n)
  {
    if (logfile == NULL)
      return *this;
    Memory s(20);
    sprintf (s, "%lu", n);
    sout (s);
    return *this;
  }

NoteCraft &NoteCraft::operator<<(ostream& (*func)(ostream&))
  { 
    if (logfile == NULL)
      return *this;
    ostrstream buffer;
    
    buffer << func;
    char *fr = buffer.str();
    fr[buffer.pcount()] = '\0';
    sout (fr);
    delete [] fr;

    // I'm going to assume this is endl and flush on it
    if (logfile == (FILE *)this) // shove out to file when you've got a full line
      fflush(stdout);
    else
      fflush(logfile); 
    return *this; 
  }

