/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef NOTECRAFT_H
#define NOTECRAFT_H

#include <stdio.h>
#include <stream.h>
#include <strstream.h>
#include "memm.h"

/* Instructions:

   NoteCraft(char *programtitle, char *filename, int minlevel = 0); 
     programtitle is for logging the program name.
     filename is the file to log to, setting this to "cout" will send log to stdout
     minlevel is the minimum loglevel you want to see messages for.

   The operator << is set up for a bunch of common types. The () function operator is
   used to set the level of this log message. IT IS IMPORTANT THAT YOU SET THE LOG LEVEL
   FOR EACH LINE. The () function operator is also used to flush a line. You must start
   each new line with a log level function call. 
   
   Note: This class does NOT inheret from ostream. But somehow it still managed to work
   well enough.
   
   Example:
   
   NoteCraft ls("test", "cout", 100);
   ls(5) << "Hi, this is the number five: " << 5 << endl; // this will not log
   ls(100) << "This is the second line, line #" << 2 << endl; // this will log
   ls << "this will make your log messy" << endl;
          
*/


/* My simple log stream */

#define NOTEDEB 0
#define NOTEDIA 100
#define NOTEINF 200
#define NOTEWRN 300
#define NOTEERR 400
#define NOTESEV 500

// handy for configcraft
#define TXT_LOGFILE "log-file"
#define TXT_LOGLEVEL "log-level"

//#define logstream NoteCraft // ain't I a weasel :-) (don't laugh, it works)

class NoteCraft
  {
    public:
      NoteCraft(char *programtitle, char *filename, int minlevel = 0); // can be 'cout'
      ~NoteCraft(void);
      
      NoteCraft &operator()(int level);
      NoteCraft &operator<<(char);
      NoteCraft &operator<<(unsigned char);
      NoteCraft &operator<<(char *);
      NoteCraft &operator<<(unsigned char *);
      NoteCraft &operator<<(int);
      NoteCraft &operator<<(unsigned int);
      NoteCraft &operator<<(long);
      NoteCraft &operator<<(unsigned long);
#ifndef no_deprecated
      NoteCraft &operator<<(ostream& (*func)(ostream&)); // for endl
#else 
      // the new way 3.2 likes it
      NoteCraft &operator<<(ostream &endl(ostream &)); // for endl
#endif
      Memory &gettitle(void) { return progname; }
      
    private:
      void sout (char *s);

      int levelset;
      int notelevel;
      Memory progname;
      FILE *logfile;
      int freshline;
    
  };

#endif

