/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <sys/time.h>
#include "memm.h"

#include "socketio.h"

  
socketio::socketio (int Infd, int Outfd, int timeout, int appropriate)
  {
    fdin = Infd;
    fdout = Outfd;
    if (timeout == 0)
      timeout = 20;
    sockettimeout = timeout; // default is 20 seconds
    socketwritetimeout = timeout; // default is 20 seconds
    need.iflip = 1; // detect endianness
    appropriated = appropriate;
    err = 0;
    xbuf = NULL;
  }

void socketio::settimeout(int timeout)
  {
    if (timeout == 0)
      timeout = 20;
    sockettimeout = timeout; 
  }

void socketio::setwritetimeout(int timeout)
  {
    if (timeout == 0)
      timeout = 20;
    socketwritetimeout = timeout; 
  }

UINT32 socketio::flip(UINT32 s)
  {
    union
      {
        char ary[4];
        UINT32 num;
      } t;
      
    t.num = s;
    char c = t.ary[0]; t.ary[0] = t.ary[3]; t.ary[3] = c;
    c = t.ary[1]; t.ary[1] = t.ary[2]; t.ary[2] = c;
    return t.num;
  }
  
int socketio::sendstr(const char *s, int len)
  {
    int size;
    int size2;
    if (len == -1)
      size = strlen(s)+1; /* Send the NT */
    else
      size = len;
    if (need.flip)
      size2 = flip(size);
    else 
      size2 = size;
    if (senddata(&size2, sizeof(size2)) == FALSE)
      return FALSE;
    return senddata((char *)s, size);
  }

int socketio::getstr(char **s)
  {
    /* THIS IS A FACTORY. YOU MUST FREEMEM THE POINTER YOU GET BACK */
    /* Your buffer better be big enough */
    int size;
    
    *s = NULL;
    if (getdata(&size, sizeof(size)) == FALSE)
      return FALSE;
    if (need.flip)
      size = flip(size);
    if (size > (1024 * 1024))
      { // assume it's bad, we never get strings bigger than a meg
        err = EINVAL; // got a better idea?
        return FALSE;
      }
                              
    *s = getmem (size); // they are sending the NT
    if (getdata(*s, size) == FALSE)
      {
        freemem (*s);
        *s = NULL;
        return FALSE;
      }
    return TRUE;
  }
  
int socketio::getstr(Memory &s)
  { // this is a better version of the above.
    int size;
    
    s = "";
    if (getdata(&size, sizeof(size)) == FALSE)
      return FALSE;
    if (need.flip)
      size = flip(size);
    if (size > (1024 * 1024))
      { // assume it's bad, we never get strings bigger than a meg
        err = EINVAL;
        return FALSE;
      }
    Memory m(size); // they are sending the NT
    if (getdata(m, size) == FALSE)
      {
        return FALSE;
      }
    s = m;
    return TRUE;
  }
  
int socketio::senddata(void *s2, int len)
  {
    fd_set fd;
    struct timeval tme;
    int offset = 0;
    ULONG starttime;
    char *s = (char *)s2;

    /* 3/19/02 All time stupid stu award. I was waiting 1millisecond instead of 1 second.
       Really bad things happen when it backs up */
    starttime = unixtime();

    while (offset < len)
      {
        if ((unixtime() - starttime) > socketwritetimeout)
          {
            //(*ls)(LSERR) << "Timed out waiting on socket write. " << socketwritetimeout << 
            //              " seconds have passed." << endl;
            err = ETIMEDOUT;
            return FALSE; /* took too long */
          }
        FD_ZERO(&fd);
        FD_SET(fdout, &fd);
        // 1/3/03 linux bug.
        tme.tv_sec = 1; 
        tme.tv_usec = 0; /* Don't hang the machine, give it some sleep time, if no data ready */
        int ret = select (fdout+1, 0, &fd, 0, &tme); // for write
        if (ret == 0)
          continue;
        if (FD_ISSET(fdout, &fd) == 0)
          continue; // our socket isn't avail to write yet
        /* Write to the socket what we can */
        starttime = unixtime();
        ret = write (fdout, s+offset, len-offset); // try and write everything
        if (ret == -1)
          {
            //(*ls)(LSERR) << "Can't write to socket. " << strerror(errno) << endl;
            err = errno;
            return FALSE; // tried to do something...
          }
        offset += ret;
      }
    return TRUE;
  }

int socketio::getdata(void *s2, int len, int takeanything)
  {
    fd_set fd;
    struct timeval tme;
    int offset;
    ULONG starttime;
    char *s = (char *)s2;
    
    /* 5/22/02 takeanything. This is for bufferedgetline. If this flag is set true
       then we will exit as soon as we get any data, if we get nothing in the timeout
       period we return false. successful return is bytes read */
    /* 3/19/02 All time stupid stu award. I was waiting 1millisecond instead of 1 second.
       Really bad things happen when it backs up */
    offset = 0;
    starttime = unixtime();

    while (offset < len)
      {
        FD_ZERO(&fd);
        FD_SET(fdin, &fd);
        // Linux trashes timeval upon return, we have to reset it constantly.
        tme.tv_sec = 1; 
        tme.tv_usec = 0; /* Don't hang the machine, give it some sleep time, if no data ready */
        int ret = select (fdin+1, &fd, 0, 0, &tme); // for read
        if (ret != 0) // something to read
          if (FD_ISSET(fdin, &fd) != 0) // and it's ours
            {
              /* Read up to the end of the packet */
              starttime = unixtime();
              ret = read (fdin, s + offset, len-offset);  // add to end
              if (ret == -1)
                {
                  //(*ls)(LSERR) << "Can't read from socket: " << strerror(errno) << endl;
                  err = errno;
                  return FALSE; // didn't get packet
                }
              if (ret == 0)
                { /* If select says yes but no data, game over */
                  int dat = 0;
                  size_t datsize = sizeof(dat);
                  ret = getsockopt(fdin, SOL_SOCKET, SO_ERROR, (char *)&dat, &datsize);
                  if ((ret != 0) || (dat == 0))
                    err = ECONNABORTED; // why no errno set?
                  else
                    err = dat;

                  return FALSE; // didn't get a packet
                }
              if (takeanything)
                return ret;
              offset += ret; // keep track of how much we got.
            } // if there's something to read
        if ((unixtime() - starttime) > sockettimeout)
          {
            //(*ls)(LSERR) << "Timed out waiting on socket read. " << sockettimeout << 
            //              " seconds have passed." << endl;
            err = ETIMEDOUT;
            return FALSE; /* took too long */
          }
      }
    return TRUE;
  }
      
int socketio::getline(Memory &line, char delimeter)
  {
    /* Cheap and lazy and slow. we read char by char. 
       If we buffer, we may screw up the stream. 
       Perhaps someday I'll make Memory a real string class and have it 
       allocate more memory on the fly rather than have realloc do it
       for me. */
    /* 3/10/00 want/need more efficient version of this. Not using addchar anymore */
    line = ""; // default if we fail for some reason
    int linesize = 1000;
    int linepos = 0;
    int growsize = 1000; // start big, so we almost never have to do this
    char *block = getmem(linesize);
    int ret = TRUE;
    
    /* we can make this faster by not using getdata 1 char at a time, 
       read and prebuffer everything until we get a crlf. the
       presumption is we stay within the buffered framework. */
    while (ret != FALSE)
      {
        ret = getdata(block+linepos, 1);
        if (ret == FALSE)
          {
            freemem(block);
            return FALSE;
          }
        if (*(block+linepos) == '\r')
          continue;
        if (*(block+linepos) == delimeter)
          break;
        linepos++;
        if (linepos+1 >= linesize) // leave room for NT
          {
            linesize += growsize;
            growsize *= 2;
            /* Don't crash because of out of memory if somebody's
               trying to kill us with too big of a single line */
            if (linesize > (1024*1024*4))
              { 
                freemem (block);
                return FALSE;
              }
            block = reallocmem(block, linesize);
          }
      }
    *(block+linepos) = '\0'; // nt it, it's a string
    line.steal(block);
    return TRUE;
  }
  
int socketio::sendline(char *line)
  {
    Memory e2(line);
    e2.addstr(CRLF);
    
    return senddata((char *)e2, strlen(e2));
  }

int socketio::bufferedgetline(Memory &line, char delimeter)
  {
    /* complex, animated and fast. We need to getline very quickly.
       The downside of this function is that once you use it, you 
       have to keep using it, unless we fix all other socketio
       functions to honor the buffer. */

    if (xbuf == NULL)
      {
        xbuf = getmem(GETLINEBUFFERSIZE);
        xbufpos = 0;
        xbufend = 0;
      }
    line = ""; // default if we fail for some reason
    int linesize = 1000;
    int linepos = 0;
    int growsize = 1000; // start big, so we almost never have to do this

    char *block = getmem(linesize);
    int more = TRUE;

    int left;
    while (more != FALSE)
      {
        left = xbufend-xbufpos;
        // copy it until we hit delim or run out of buffer
        while (left > 0)
          {
            char *here = block+linepos;
            *here = *(xbuf + xbufpos);
            left--;
            xbufpos++;
            
            if (*here == delimeter) // we're at end of line
              {
                more = FALSE;
                break; 
              }
            if ((*here == '\r') && (delimeter == '\n')) // this is different from getline
              continue; // go to next char
    
            linepos++; // advance write buffer and check for overflow
            if (linepos+1 >= linesize) // leave room for NT
              {
                linesize += growsize;
                growsize *= 2;
                /* Don't crash because of out of memory if somebody's
                   trying to kill us with too big of a single line */
                if (linesize > (1024*1024*4))
                  { 
                    freemem (block); // xbuf is in an okay, if out of sync state
                    return FALSE;
                  }
                block = reallocmem(block, linesize);
              }
          } // end of copying from buffered input

        if (more)
          {
            // we need more data, read from the stream 
            xbufpos = 0;
            xbufend = getdata(xbuf, GETLINEBUFFERSIZE, TRUE); // come back when you get ANYTHING
            if (xbufend == FALSE)
              {
                freemem (block);
                return FALSE;
              }
          }
      } // end while we're still looking for delim

    *(block+linepos) = '\0'; // nt it, it's a string
    line.steal(block);
    return TRUE;
  }

