/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ZARY_H
#define ZARY_H

#include "memm.h"
#include "entry.h"

/* A simple array class. Like zlist, but it maintains position when you delete things.
   destruction it acts like slist.clearAndDestroy();
   Example:
     zary x;
     stringentry *s = new stringentry("Hi from Stu.");
     int pos = x.Add(s); // takes ownership
     cout << (char *)(((stringentry *)x.GetItem(pos))->getstr());
*/

// char is smaller than int
#define ZFLAG char 

class zary
  {
    // somewhat abstract base list class
    public:
      zary()
        {
          items = (entry **)getmem(1); // always something
          used = (ZFLAG *)getmem(1); // always something
          itemscount = 0;
          memitemscount = 0;
        }
      virtual ~zary()
        {
          int lp;
          for (lp = 0; lp < itemscount; lp++)
            {
              if (used[lp] != FALSE)
                delete(items[lp]); // we own, and they were newed
            }
          freemem((char *)items);
          freemem((char *)used);
        }
      virtual int Add (entry *item); // returns position
      virtual void Del (int id);
      virtual void DelPos (int pos);  // marks it free
      virtual int ListCount(void) 
        { 
          int count = 0;
          int lp;
          for (lp = 0; lp < itemscount; lp++)
            if (used[lp] != FALSE)
              count++;
          return count; 
        }
      virtual int FindPos (int itemid);
      virtual entry *GetItem(int pos) { return items[pos]; }
      virtual int MaxItems(void) { return itemscount; } // high water mark
      virtual void WipeList(void)
        {
          int lp;
          for (lp = 0; lp < itemscount; lp++)
            if (used[lp] != FALSE)
              delete(items[lp]); // we own, and they were newed
          freemem((char *)items);
          items = (entry **)getmem(1); // always something
          itemscount = 0;
          memitemscount = 0;
        }

    protected:
      entry **items;
      ZFLAG *used;
      int itemscount;
      int memitemscount;
  };


#endif

