/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include "zlist.h"

int zlist::Add (entry *item)
  {
    // we take ownership of this item
    
    if (itemscount >= memitemscount)
      {
        const int memjump = 5; // make room for 5 at a time
        /* Make more room in the list */
        entry **temp;
        temp = (entry **)getmem(sizeof(item) * (memitemscount + memjump));
        memcpy (temp, items, sizeof(item) * memitemscount);
        memitemscount += memjump;
        freemem((char *)items);
        items = temp;
      }
    /* Add item to the end of the list. Items are NEWed, lists are getmemed */
    items[itemscount] = item; // we own it
    // set the id
    int iid;
    if (itemscount == 0)
      iid = 500; // for debugging
    else
      iid = items[itemscount - 1]->get_itemid() + 1;
    items[itemscount]->set_itemid(iid);
    itemscount++;
    return iid;
  }

int zlist::FindPos (int itemid)
  {
    int lp;
    for (lp = 0; lp < itemscount; lp++)
      if (items[lp]->get_itemid() == itemid)
        return lp;
    return -1;
  }
  
int zlist::FindPosByString(char *s)
  { 
    // subclass Finder() in your entry class to make this work
    int lp;
    for (lp = 0; lp < itemscount; lp++)
      if (strcmp(items[lp]->Finder(), s) == 0)
        return lp;
    return -1;
  }
  
void zlist::Del (int id)
  {
    /* Find this item's id */
    int lp = FindPos (id);
    if (lp != -1)
      DelPos(lp);
  }

void zlist::DelPos (int pos)
  {
    if (pos != -1)
      {
        delete items[pos];
        while (pos < itemscount)
          {
            items[pos] = items[pos+1];
            pos++;
          }
        itemscount--;
      }
  }

void zlist::Sort (int (*comparefunction)(const void*, const void*))
  {
    // it's so convienent that we have a pile of pointers, just 
    // like qsort wants.
    qsort (items, itemscount, sizeof(entry *), comparefunction);
  }

