/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ZLIST_H
#define ZLIST_H

#include "memm.h"
#include "entry.h"

/* A simple list class. Like RWSListCollectables. Order is determined by
   insertion order. This Object Owns Everything Passed To It. Upon 
   destruction it acts like slist.clearAndDestroy();
   Example:
     zlist x;
     stringentry *s = new stringentry("Hi from Stu.");
     x.Add(s); // takes ownership
     cout << (char *)(((stringentry *)x.GetItem(0))->getstr());
*/


class zlist
  {
    // somewhat abstract base list class
    public:
      zlist()
        {
          items = (entry **)getmem(1); // always something
          itemscount = 0;
          memitemscount = 0;
        }
      virtual ~zlist()
        {
          int lp;
          for (lp = 0; lp < itemscount; lp++)
            delete(items[lp]); // we own, and they were newed
          freemem((char *)items);
        }
      virtual int Add (entry *item);
      virtual void Del (int id);
      virtual void DelPos (int pos);
      virtual int ListCount() { return itemscount; }
      virtual int FindPos (int itemid);
      virtual int FindPosByString(char *s);
      virtual entry *GetItem(int pos) { return items[pos]; }
      virtual void SwapItem (int pos1, int pos2) 
        { 
          // this is for webmail
          entry *temp = items[pos1];
          items[pos1] = items[pos2];
          items[pos2] = temp;
        }
      virtual void Sort (int (*comparefunction)(const void*, const void*));
      virtual void WipeList(void)
        {
          int lp;
          for (lp = 0; lp < itemscount; lp++)
            delete(items[lp]); // we own, and they were newed
          freemem((char *)items);
          items = (entry **)getmem(1); // always something
          itemscount = 0;
          memitemscount = 0;
        }

    protected:
      entry **items;
      int itemscount;
      int memitemscount;
  };


class stringentry : public entry
  {
    public:
      stringentry (char *s1)
        { 
          str = s1;
        }
      stringentry (Memory &m1)
        { 
          str = m1;
        }
      Memory &getstr(void) { return str; }

    private:
      Memory str;
  };


#endif

