/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* This version to be used with tragic 5/14/98 */

/* Neat little set of C routines that do DBMS with flat files
   with no indexing. It'll be quick because the files aren't too big.
   Possibly the way we should have done it from the very beginning. */

/* The file format will be a simple, header of your choosing, a CRLF (2 bytes)
   then one record after the other of [recordsize] length, followed by a
   CRLF (for readability when we have to do support over the phone and
   we want to be able to pull this up in an editor. */

/* I'm going to get me some good reuse gas money out of this :) */

/* 2/26/97 Adding a '!' to the end of each line so it's easier to pull
   up and play with in an editor */

/* This program is not very unix centric, all lines end in \r\n */

/* 12/29/97 Adding bin tree index. -1 denotes leaf */
/* 12/31/97 Fix index for delete last record */
/* 12/31/97 Added reindex */
/* 1/5/98   Adding blobs */
/* 5/14/98  Minor config changes for tragic */
/* 8/30/00  Interesting problem. If you have a file that mixes blobs 
            and non blob records, you can cause corruption by deleting
            a non blob record. If a blob record gets moved it might
            get indexed incorrectly becuase the keylength is 5 bigger
            but nobody knows that, so to hack fix that, all keys are
            5 bigger all the time (except for seeking non blob) and the
            recordsize better be at least 5 bytes :-) What a piece of
            shit this turned into, eh? */
/* 1/5/2001 Well, what I said was true there, but I didn't seem to
            actually implement the part about the "except for seeking
            non blob." I think what I'm going to do is put back the
            delta key stuff, and just bump it on the delete and addindex */
/* 1/9/2001 By some amazing oversight, I left out the passing of the key
            length. So you can't seek on a partial key. I'm fixing that
            now. It's in the fc and the flatserver, just not here. 
            I'm pretty sure we don't want to allow variable key length
            for blobs so even if they get passed, they will be
            ignored, actually the flatserver will make sure the
            file definition length is used, not what is passed. 
            Actually the only thing that can partial seek
            is seek and delete, update, insert and blobs have to
            be full key. */

#include <stdio.h>
#include <malloc.h>
#include <memory.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <osconfig.h>  
#include <../Q/memm.h>
#include "flat.h"


/* For now the file handle will be the text of the filename and we will
   open and close it each time. If this proves to be too slow, we'll leave it
   open */


/* Internal use only */
long flatfindrec (FLATHAND fhandle, char *key, int keylen, char *out);
void updateindex(FLATHAND fhandle, long recno);
long followright (FLATHAND ff, long rootrec);
void delindex(FLATHAND fhandle, long recno);
void addindex(FLATHAND fhandle, long recno);
int keysize (FLATHAND f);
long flatfindrecindex (FLATHAND fhandle, char *key, int keylen, char *out);
long bestparent(FLATHAND fhandle, char *srch, int keylen, int findmatch);
void setrootnode (FLATHAND fhandle, long recno);
long getrootnode (FLATHAND fhandle);
int blob5seek (FLATHAND fhandle, char *key, char *out);
int flatseek2 (FLATHAND fhandle, char *key, int keylen, char *out);
int flatappend2 (FLATHAND fhandle, char *in);
int flat5append (FLATHAND fhandle, char *in);


#define NULL0 0
int deltakeysize = 0; /* the blob hack */

FLATHAND flatopen (char *file, int headersize, int recordsize, int index, int keysize)
  {
    /* For now we're using fopen and friends... */
    FLATHAND f;
    size_t sz;
    char *buf;

    sz = sizeof (struct tagflathand);
    f = (FLATHAND)getmem (sz);
    /* unix ignores 3rd param */
    f->f = osopen (file, FLATMODE, FLATPARAM); /* read/write binary */
    if (f->f == -1)
      { /* uh oh */
        /* error2 ("Error creating/openeing: ", file);
           error2 ("RC: ", strerror(errno));
 */
        freemem (f);
        return NULL;
      }
/*     debug ("filehandle: %d\n", f->f); */
    f->headersize = headersize;
    f->recordsize = recordsize;
    f->indexsize = keysize;
    if (index != FALSE)
      {
        /* open/create index file */
        char iname[200];
        char *dpos;
        strcpy (iname, file);
        dpos = strrchr (iname, '.');
        if (dpos == NULL)
          strcat (iname, ".ind");
        else
          strcpy (dpos, ".ind"); /* this will flub if ..\dirname has . and file doesn't */

        f->index = flatopen (iname, sizeof(sheaderIndex), sizeof(sIndex), FALSE, 4); /* index doesn't have index 4 bytes for root node */
      }
    else
      f->index = NULL;
    /* 8/30/00 safeguard, if no header has been written, write one. it messes things
       up if you don't. */
    buf = getmem(headersize);
    if (getheader (f,  buf) != 0)
      {
        long ind = -1;
        memset (buf, 0, headersize);
        /* hack for creating an index file */
        if (headersize == 4)
          memcpy (buf, &ind, 4); // root node is empty
        setheader(f, buf);
      }
    freemem(buf);
    return f;
  }

void flatclose (FLATHAND *f)
  {
    int ret;
    if ((*f)->index != NULL0)
      flatclose (&((*f)->index)); /* location of the index filehandle */

    ret = osclose ((*f)->f);
/*    if (ret != 0)
      error2 ("Error closing file in flatclose, RC: ", strerror(errno));
*/
    freemem((char *)*f);
    *f = NULL0;
  }

int getheader (FLATHAND fhandle, char *out)
  {
    /* Return the header from the file in 'out'.
       0 = success, 1 = FAIL.
    */
    int ret;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    /* Go to top of file */
    if (lseek (handle, 0, SEEK_SET) < 0)
      return 1;
    ret = read (handle, out, fhandle->headersize);
    if (ret < 1)
      return 1;
    return 0;
  }

int setheader (FLATHAND fhandle, char *in)
  {
    /* Set the header in the file in from 'in'.
       0 = success, 1 = FAIL.
    */
    int ret;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;
    /* Go to start of file */
    if (lseek (handle, 0, SEEK_SET) < 0)
      return 1;
    ret = write (handle, in, fhandle->headersize);
    if ((ret > 0) || (fhandle->headersize == 0))
      write (handle, "\r\n", 2); /* write out the CRLF */
    if ((ret == 0) && (fhandle->headersize != 0))
      return 1;
    return 0;
  }

int flatread (FLATHAND fhandle, long recno, char *out)
  {
    /* Given a filehandle, a record number, and a place to put it,
       this guy will read that record into 'out'.
       Record numbers are 0 based.
       0 = success, 1 = FAIL.
    */
    int ret;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    /* I know this is a horrid waste of time, but it will avoid bugs */
    if ((recno < 0) || (recno >= flatrecno(fhandle))) // must be in valid range
      return 1; 
    
    /* Go to correct position in file, header+crlf, record+crlf  */
    if (lseek (handle, fhandle->headersize + 2 + (((fhandle->recordsize)+3) * recno), SEEK_SET) < 0)
      return 1;
    ret = read (handle, out, (fhandle->recordsize));
    if (ret < 1)
      return 1;
    return 0;
  }

int flatwrite (FLATHAND fhandle, long recno, char *in)
  {
    /* Given a filehandle, a record number, and a source buffer,
       this guy will write that record into 'out'.
       Record numbers are 0 based.
       0 = success, 1 = FAIL.
    */
    int ret;
    int handle;

    if (fhandle == NULL0)
      return 1;

    /* You can't use flat write if you have an index, but we
       can't eliminate it because the delete uses it. */

    handle = fhandle->f;
    /* Go to correct position in file, header+crlf, record+crlf  */
    if (lseek (handle, (fhandle->headersize) + 2 + (((fhandle->recordsize)+3) * recno), SEEK_SET) < 0)
      return 1;

    ret = write (handle, in, (fhandle->recordsize));
    if (ret > 0)
      write (handle, "!\r\n", 3); /* write out the EOL marker and CRLF */
    else
      return 1;
    return 0;
  }

long flatseekrec (FLATHAND fhandle, char *key, int keylen)
  { /* returns record number or -1 if not found */
    long recno;
    char *tbuf;

    if (fhandle == NULL0)
      return -1;
    if (fhandle->index == NULL0)
      return -1;
    
    tbuf = getmem ((fhandle->recordsize)+5); /* need temp buffer space */

    /* do bin search */
    recno = flatfindrecindex (fhandle, key, keylen, tbuf);

    freemem ((void *)tbuf);
    return recno;
  }

int flatseek (FLATHAND fhandle, char *key, int keylen, char *out)
  {
    deltakeysize = 0; 
    return flatseek2(fhandle, key, keylen, out);
  }

int blob5seek (FLATHAND fhandle, char *key, char *out)
  {
    int ret;
    deltakeysize = 5; /* the 'B' and the long seq no */
    ret = flatseek2(fhandle, key, fhandle->indexsize, out);
    deltakeysize = 0;
    return ret;
  }

int flatseek2 (FLATHAND fhandle, char *key, int keylen, char *out)
  {
    /* Given a filehandle, a key, and a place to put it,
       this guy will find the key and read that record into 'out'.
       0 = success, 1 = FAIL, 2 = key not found.
    */
    int ret;
    long recno;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    if (fhandle->index != NULL)
      {
        /* do bin search */
        recno = flatfindrecindex (fhandle, key, keylen, out);
      }
    else
      {
        /* Do the raw search */
        recno = flatfindrec (fhandle, key, keylen, out);
      }
    if (recno == -1)
      return 2; /* not found */

    /* Go to correct position in file, header+crlf, record+crlf  */
    if (lseek (handle, (fhandle->headersize) + 2 + (((fhandle->recordsize)+3) * recno), SEEK_SET) < 0)
      return 1;
    ret = read (handle, out, (fhandle->recordsize));
    if (ret < 1)
      return 1;
    return 0;
  }

int flatupdate (FLATHAND fhandle, char *key, char *in)
  { /* full key required here 1/9/01 */
    /* Look for key, if found, update the record from in.
       0 = success, 1 = fail, 2 = not found
    */
    int ret;
    long recno;
    char *tbuf;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    /* Do the raw search */
    tbuf = getmem ((fhandle->recordsize)+5); /* need temp buffer space */
    if (fhandle->index == NULL)
      recno = flatfindrec (fhandle, key, fhandle->indexsize, tbuf);
    else
      recno = flatfindrecindex (fhandle, key, fhandle->indexsize, tbuf);

    freemem ((void *)tbuf);
    if (recno == -1)
      return 2; /* not found */

    /* Go to correct position in file, header+crlf, record+crlf  */
    if (lseek (handle, (fhandle->headersize) + 2 + (((fhandle->recordsize)+3) * recno), SEEK_SET) < 0)
      return 1;
    ret = write (handle, in, (fhandle->recordsize));
    if (ret > 0)
      write (handle, "!\r\n", 3); /* write out the CRLF */
    else
      return 1;
    /* A nice optomization would be to skip this step if we notice
       the key doesn't change. 1/3/03 */
    updateindex(fhandle, recno); /* erase this index entry and re-add in right spot */
    return 0;
  }


int flatappend (FLATHAND fhandle, char *in)
  {
    deltakeysize = 0;
    return flatappend2(fhandle, in);
  }

int flat5append (FLATHAND fhandle, char *in)
  {
    int ret;

    deltakeysize = 5;
    ret = flatappend2(fhandle, in);
    deltakeysize = 0;
    return ret;
  }

int flatappend2 (FLATHAND fhandle, char *in)
  {
    /* This one's kinda interesting. We have no idea how big the file is,
       so we have to figure it out somehow.
       Normally, I'd romp to the end of the file, but that would take
       a long time on a big file, so I'm going to figure out the record
       by the file size, and if it doesn't line up, I'm going to
       chop off the last record.
       If file is buggy, you will lose data using this function
       0 = success, 1 = fail */

    int ret;
    long pos;
    int handle;
    long recno;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    /* Figure out the last record number */
    if (lseek (handle, 0, SEEK_END) < 0)
      return 1;

    pos = tell (handle);
    if (pos == -1)
      return 1;
    pos -= ((fhandle->headersize)+2); /* skip the header */
    /* Now see if we're on a record boundary */
    if ((pos % ((fhandle->recordsize)+3)) != 0)
      {
        /* we're about to lose data */
        pos = (pos / ((fhandle->recordsize)+2)) * ((fhandle->recordsize)+3);
      }

    recno = pos / ((fhandle->recordsize)+3); /* figure out the recno for index */

    pos += (fhandle->headersize)+2; /* goto the place in the file +CRLF */
    if (lseek (handle, pos, SEEK_SET) < 0)
      return 1;
    ret = write (handle, in, (fhandle->recordsize));
    if (ret > 0)
      write (handle, "!\r\n", 3); /* write out the CRLF */
    else
      return 1;

    addindex(fhandle, recno); /* add in right spot */
    return 0;
  }

int flatreaddel (FLATHAND fhandle, long recno)
  {
    /* Delete this record number, by moving the last record to this
       record position and shorteneing the file
       0 = success, 1 = fail, 2 = not found */
    long lrpos;
    char *buf;
    long zrecno;
    int ret;
    int od;

    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;

    /* Get position of last record */
    if (lseek (handle, 0, SEEK_END) < 0)
      return 1;

    lrpos = tell (handle);
    if (lrpos == -1)
      return 1;
    lrpos -= ((fhandle->headersize)+2); /* skip the header */
    /* Get highest record number */
    zrecno = lrpos / ((fhandle->recordsize)+3); /* This will round down if not aligned */

    zrecno--; /* go back to start of last good record */

    if (recno > zrecno) /* You can't delete a record past the last one */
      return 1;

    /* delete fold index entry for recno, it's about to get wiped */
    /* 1/5/2001 bigindex delete fix */
    od = deltakeysize;
    deltakeysize = 5;
    delindex(fhandle, recno); /* remove this entry */
    deltakeysize = od;

    buf = getmem ((fhandle->recordsize)+5);
    if (flatread (fhandle, zrecno, buf) != 0)
      {
        freemem ((void *)buf);
        return 1;
      }
    /* Now write it in new spot */
    flatwrite (fhandle, recno, buf);
    freemem (buf);
    /* The index makes this confusing. We delete the old
       record's index record, then we delete zrecno, the one we're
       copying from.
       and let the add take care of adding it in the right place */
    /* In the case of deleteing the last record we copy the last record
       over the last record and shorten the file. No net effect, but
       the index scheme will mess up because we'll del #3, then del #3
       then add #3. The second del doesn't do anything so we end up
       with the same number of indexes we started with. So for the
       last record don't delete or add the index again */
    if (zrecno != recno)
      {
        od = deltakeysize; /* 1/5/2001 */
        deltakeysize = 5;
        delindex (fhandle, zrecno);
        addindex(fhandle, recno); /* add this new record */
        deltakeysize = od;
      }

    /* Now shorten the file. */
    /* I dunno, write an eof? */
    ret = chsize(handle, zrecno*((fhandle->recordsize)+3) + ((fhandle->headersize)+2));
    if (ret < 0)
      return 1;
    return 0;
  }

int flatseekdel (FLATHAND fhandle, char *key, int keylen)
  {
    /* Seek on the key and do a flat readdel with it
       0 = success, 1 = fail, 2 = not found */
    long recno;
    char *tbuf;
    int handle;

    if (fhandle == NULL0)
      return 1;
    handle = fhandle->f;
    /* Do the raw search */
    tbuf = getmem ((fhandle->recordsize)+5); /* need temp buffer space */
    if (fhandle->index != NULL)
      {
        /* do bin search */
        recno = flatfindrecindex (fhandle, key, keylen, tbuf);
      }
    else
      {
        /* Do the raw search */
        recno = flatfindrec (fhandle, key, keylen, tbuf);
      }

    freemem ((void *)tbuf);
    if (recno == -1)
      return 2; /* not found */

    return flatreaddel(fhandle, recno); /* this takes care of index */
  }

int flat5seekdel (FLATHAND fhandle, char *key)
  {
    /* Seek on the key and do a flat readdel with it
       0 = success, 1 = fail, 2 = not found */
    /* works with the extended key length of the blob */
    long recno;
    char *tbuf;
    int handle;
    int ret;

    handle = fhandle->f;

    tbuf = getmem ((fhandle->recordsize)+5); /* need temp buffer space */
    deltakeysize = 5;
    recno = flatfindrecindex (fhandle, key, fhandle->indexsize, tbuf);
    deltakeysize = 0;

    freemem ((void *)tbuf);
    if (recno == -1)
      return 2; /* not found */

    /* The deleted and add for the index in here also have to refer
       to the large key */
    deltakeysize = 5;
    ret = flatreaddel(fhandle, recno); /* this takes care of index */
    deltakeysize = 0;
    return ret;
  }

long flatfindrec (FLATHAND fhandle, char *key, int keylen, char *out)
  {
    /* do search on open file, return record number or
       -1 if not found */
    long i;
    int ret;
    int f;

    f = fhandle->f;

    /* We don't know the number of records, so we go till end */
    i = 0;
    while (f != 0) /* while (1) */
      {
        if (lseek (f, (fhandle->headersize) + 2 + (((fhandle->recordsize)+3) * i), SEEK_SET) < 0)
          return -1;
        ret = read (f, out, (fhandle->recordsize));
        if (ret < 1)
          return -1;
        /* check for key */
        if (memcmp (key, out, keylen) == 0)
          return i;
        i++;
      }
    return 0;
  }

long flatfindrecindex (FLATHAND fhandle, char *key, int keylen, char *out)
  {
    /* bin search */
    long ret;
    ret = bestparent(fhandle, key, keylen, TRUE); /* do bin search */
    /* Store record in out */
    if (ret != -1)
      flatread (fhandle, ret, out);

    return ret;
  }

long flatrecno (FLATHAND fhandle)
  {
    /* Return the number of records in the file. Same mentality that
       append uses to figure it out
       returns -1 if error */

    long pos;
    int handle;

    if (fhandle == NULL0)
      return -1;
    handle = fhandle->f;

    /* Figure out the last record number */
    if (lseek (handle, 0, SEEK_END) < 0)
      return -1;

    pos = tell (handle);
    if (pos == -1)
      return -1;
    pos -= ((fhandle->headersize)+2); /* skip the header */
    /* Now see if we're on a record boundary */
    if ((pos % ((fhandle->recordsize)+3)) != 0)
      {
        /* we're about to lose data */
        pos = (pos / ((fhandle->recordsize)+2)) * ((fhandle->recordsize)+3);
      }

    return (pos / ((fhandle->recordsize)+3));
  }

long getrootnode (FLATHAND fhandle)
  {
    sheaderIndex s;
    int ret;
    
    if (fhandle == NULL)
      return -1;
    if (fhandle->index == NULL)
      return -1;

    ret = getheader (fhandle->index, (char *)&s);
    if (ret != 0) /* file empty? */
      {
        setrootnode(fhandle, -1);
        return -1;
      }
    return s.rootnode;
  }

void setrootnode (FLATHAND fhandle, long recno)
  {
    /* This is just set the header */
    sheaderIndex s;
    s.rootnode = recno;
    if (fhandle == NULL)
      return;
    if (fhandle->index == NULL)
      return;
    setheader (fhandle->index, (char *)&s);    
  }
  
int icompare (FLATHAND fhandle, char *s1, char *s2, int keysize)
  {
    /* I'd like to make this search based on the length of s1, the
       search key, but that assumes strings, and we're doing binary,
       so don't forget that. If you're checking strings, you have to
       pad spaces and match length. So there. */
    /* 1/9/01 Removed the keysize function, it only got called here.
       We deal with delta keysize off of what we're told to check.
       For seeks and [seek for] delete it might be short, but for blobs
       and update it will be the full keysize. */
    /* return >0 if s1 comes after s2 */
    int ret;
    ret = memcmp (s1, s2, keysize + deltakeysize);
    if (ret > 0)
      return 1;
    if (ret == 0)
      return 0;
    return -1;
  }

long bestparent(FLATHAND fhandle, char *srch, int keylen, int findmatch)
  {
    /* srch is key to find a kid for.
       return the record number of the guy who would make the
       best parent for this key */
    /* Record # 0 is always first record/root of tree */
    /* 1/10/01 Since yesterday we decided to support partial
       key seeks, we have to solve the problem of not just
       getting to a good record, but getting to the FIRST
       good record. So when we find something, try following
       the left tree until we don't match anymore. */
    sIndex k;
    char *skey;
    long root;
    int keepgoing = TRUE;
    int bestfound = -1; // not found flag, and flag to stop when we get to a bad left 1/10/01

    skey = getmem(fhandle->recordsize + 5);

    root = getrootnode(fhandle);
    while (keepgoing == TRUE)
      {
        long nroot;
        int ret;

        if (flatread (fhandle, root, skey) != 0)
          {
            freemem (skey);
            return bestfound;
          }
        if (flatread (fhandle->index, root, (char *)&k) != 0)
          {
            freemem (skey);
            return bestfound;
          }
        ret = icompare(fhandle, srch, skey, keylen);

        if (findmatch != 0) /* so it's like seek */
          {
            if (bestfound != -1) // if we have something good
              if (ret != 0) // and what we have now is worse
                {
                  freemem(skey); // return it
                  return bestfound; 
                }
              else
                bestfound = root; // this is better, keep this and continue
            else // if bestfound is not set yet
              if (ret == 0)
                {
                  bestfound = root; // this is our first good match
                }
            // continue. If we are here we have a match and will follow left to get a possibly better one
          } // findmatch != 0
        if (ret > 0)
          nroot = k.right;
        else
          nroot = k.left;

        if (nroot == -1) /* We hit a leaf */
          if (findmatch != 0)
            {
              freemem (skey);
              return bestfound; /* either not found or best so far */
            }
          else
            {
              freemem (skey);
              return root;
            }
        root = nroot;
      }
    return 0;
  }

void addindex(FLATHAND fhandle, long recno)
  {
    /* We ensure that the number of index records and the number of
       actual records stay in sync because the callers do things
       at the right times. */

    /* Find our new parent, set our parent to that and set their kid to me */
    FLATHAND ff;
    long par;
    char *k;
    char *park;
    sIndex ikey;

    ff = fhandle->index;
    if (ff == NULL)
      return;

    /* Take care of first record case */
    if (getrootnode(fhandle) == -1) /* We're the first */
      {
        ikey.left = -1;
        ikey.right = -1;
        ikey.parent = -1;
        flatwrite (ff, recno, (char *)&ikey);
        setrootnode(fhandle, recno);
        return;
      }

    k = getmem(fhandle->recordsize + 5);
    park = getmem(fhandle->recordsize + 5);

    if (flatread (fhandle, recno, k) != 0)
      {
        freemem(k);
        freemem(park);
        return;
      }

    /* Find the best parent to add to */
    par = bestparent(fhandle, k, fhandle->indexsize, FALSE); /* Don't seek, find a parent */
    if (par == -1)
      {
        freemem(k);
        freemem(park);
        return;
      }
    if (flatread (fhandle, par, park) != 0)
      {
        freemem(k);
        freemem(park);
        return;
      }

    /* Write my index. I'm a leaf and all I know is my parent */
    ikey.left = -1;
    ikey.right = -1;
    ikey.parent = par;
    flatwrite (ff, recno, (char *)&ikey); /* write the index record for me */

    /* Read the parent's index record in */
    if (flatread (ff, par, (char *)&ikey) != 0)
      {
        freemem(k);
        freemem(park);
        return;
      }

    if (icompare (fhandle, park, k, fhandle->indexsize) > 0)
      ikey.left = recno;
    else
      ikey.right = recno;
    /* write it out */
    flatwrite (ff, par, (char *)&ikey);
    freemem(k);
    freemem(park);
  }

void delindex(FLATHAND fhandle, long recno)
  {
    /* Find our new parent, set our parent to that and set their kid to me */
    FLATHAND ff;
    long par;
    sIndex ikey;

    ff = fhandle->index;
    if (ff == NULL)
      return;

    if (flatread (ff, recno, (char *)&ikey) != 0)
      return;

    /* Check the simplest case. no kids */
    if ((ikey.left == -1) && (ikey.right == -1))
      {
        /* tell my parent that I'm gone */
        long par;
        par = ikey.parent;
        if (par == -1)
          {  /* deleting root node */  
            setrootnode(fhandle, -1);
            return; 
          }
        if (flatread (ff, par, (char *)&ikey) != 0)
          return;
        /* Whichever kid we are, snub me */
        if (ikey.left == recno)
          ikey.left = -1;
        if (ikey.right == recno)
          ikey.right = -1;
        flatwrite (ff, par, (char *)&ikey);
        return;
      }

    /* next easiest case, 1 kid */
    if ((ikey.left == -1) || (ikey.right == -1))
      {
        /* tell my parent that their kid is my kid */
        long mykid;

        if (ikey.left == -1)
          mykid = ikey.right;
        else
          mykid = ikey.left;

        par = ikey.parent;
        if (par == -1)
          { /* If I was the root node, my kid becomes the root node */
            setrootnode(fhandle, mykid);
            /* And he has no parent */
            if (flatread (ff, mykid, (char *)&ikey) != 0)
              return;
            ikey.parent = -1; /* flag him as root */
            flatwrite (ff, mykid, (char *)&ikey);
            return; 
          }
        if (flatread (ff, par, (char *)&ikey) != 0)
          return;
        /* Whichever kid we are, snub me */
        if (ikey.left == recno)
          ikey.left = mykid;
        if (ikey.right == recno)
          ikey.right = mykid;
        flatwrite (ff, par, (char *)&ikey);

        /* And tell my kid who his new parent is */
        /* Read mine back in */
        if (flatread (ff, mykid, (char *)&ikey) != 0)
          return;
        ikey.parent = par; /* now that we know it */
        flatwrite (ff, mykid, (char *)&ikey);

        return;
      }
    else
      {
        /* if we're here, this is the worst case, we have 2 kids. */
        /* the trick is to get my immediate sucessor or predecessor
           and put them in my spot.
           The predecessor is found by going left once and going right
           till you're at a leaf. Kill him and he's our parent's new
           kid. */

        long predrec;

        predrec = followright (ff, ikey.left);
        if (predrec == -1)
          return; /* problem, bail */
        /* He's either a leaf or a single child, kill him */
        delindex (fhandle, predrec);

        /* The oddest case is where the pred is our kid, have to delete
           him (we just did) and then reread our index record because
           it just got changed */
        if (flatread (ff, recno, (char *)&ikey) != 0)
          return;

        /* ikey is our old info, our pred now has to take over our kids */
        flatwrite (ff, predrec, (char *)&ikey); /* write OUR index into HIS record */

        /* Tell the origional record's 2 kids who their
           new parent is. Since we know we have 2 kids we shouldn't
           have to check except for the bad case that our pred is also
           our kid, and he may have gone away just now */
        if (ikey.right != -1)
          {
            sIndex ik;
            flatread (ff, ikey.right, (char *)&ik);
            ik.parent = predrec;
            flatwrite (ff, ikey.right, (char *)&ik);
          }
        if (ikey.left != -1)
          {
            sIndex ik;
            flatread (ff, ikey.left, (char *)&ik);
            ik.parent = predrec;
            flatwrite (ff, ikey.left, (char *)&ik);
          }

        /* and Now tell our parent that his kid is our pred */
        par = ikey.parent;
        if (par == -1)
          { /* if we're rootnode, our predecessor becomes the root node */
            setrootnode(fhandle, predrec);
          }
        else
          {
            if (flatread (ff, par, (char *)&ikey) != 0)
              return;
            /* Whichever kid we WERE, snub me */
            if (ikey.left == recno)
              ikey.left = predrec;
            if (ikey.right == recno)
              ikey.right = predrec;
            flatwrite (ff, par, (char *)&ikey);
          }
      }
    return;
  }

long followright (FLATHAND ff, long rootrec)
  {
    /* Return the record number at the right side leaf end of rootrec */
    sIndex k;
    int keepgoing = TRUE;

    if (rootrec == -1)
      return -1;
    while (keepgoing == TRUE)
      {
        if (flatread (ff, rootrec, (char *)&k) != 0)
          return -1;
        if (k.right == -1)
          return rootrec;
        rootrec = k.right;
      }
    return 0;
  }

long followleft (FLATHAND ff, long rootrec)
  {
    /* Return the record number at the left side leaf end of rootrec */
    sIndex k;
    int keepgoing = TRUE;

    if (rootrec == -1)
      return -1;
    while (keepgoing == TRUE)
      {
        if (flatread (ff, rootrec, (char *)&k) != 0)
          return -1;
        if (k.left == -1)
          return rootrec;
        rootrec = k.left;
      }
    return 0;
  }

void updateindex(FLATHAND fhandle, long recno)
  {
    /* updating is just deleting the entry and readding it */
    delindex (fhandle, recno);
    addindex (fhandle, recno);
  }

void reindex(FLATHAND fhandle)
  {
    long ret;
    long lp;

    /* build it from scratch */
    if (fhandle == NULL)
      return;
    if (fhandle->index == NULL)
      return;
      
    /* First shrink the index file */
    chsize(fhandle->index->f, 0);

    /* make the tree empty */
    setrootnode(fhandle, -1);
    /* Romp through the whole file adding index records */
    
    deltakeysize = 5; /* 8/29/00 in case there's any blobs in this file, we have to include the extra key size */
    ret = flatrecno (fhandle);
    for (lp = 0; lp < ret; lp++)
      addindex(fhandle, lp);
    deltakeysize = 0;
  }

void dumpindex(FLATHAND fhandle)
  {
    long ret;
    long lp;
    sIndex ikey;
    char *g;

    g = getmem (fhandle->recordsize+5);
    
    ret = flatrecno (fhandle);
    for (lp = 0; lp < ret; lp++)
      {
        flatread (fhandle, lp, g);
        flatread (fhandle->index, lp, (char *)&ikey);
        printf ("Rec, Parent, left, right: %5ld %5ld %5ld %5ld  %s\n", lp, ikey.parent, ikey.left, ikey.right, g);
      }
    freemem(g);
  }

void dumpkey(FLATHAND fhandle)
  {
    long lp;
    char *s;
    
    s = getmem (fhandle->recordsize+5);
    lp = firstrec(fhandle);
    while (lp != -1)
      {
        flatread (fhandle, lp, s);
        *(s+fhandle->recordsize) = '\0';
        printf ("%s\n", s);
        lp = nextrec(fhandle, lp);
      }
    freemem(s);
  }

void dumpreverse(FLATHAND fhandle)
  {
    long lp;
    char *s;
    
    s = getmem (fhandle->recordsize+5);
    lp = lastrec(fhandle);
    while (lp != -1)
      {
        flatread (fhandle, lp, s);
        *(s+fhandle->recordsize) = '\0';
        printf ("%s\n", s);
        lp = prevrec(fhandle, lp);
      }
    freemem(s);
  }

/* Useful key based routines */

long firstrec (FLATHAND fhandle)
  {
    /* scoot down the left side */
    FLATHAND ff;

    if (fhandle == NULL)
      return -1;
    ff = fhandle->index;
    if (ff == NULL)
      return -1;
    return followleft (ff, getrootnode(fhandle));
  }
  
long lastrec (FLATHAND fhandle)
  {
    /* scoot down the left side */
    FLATHAND ff;

    if (fhandle == NULL)
      return -1;
    ff = fhandle->index;
    if (ff == NULL)
      return -1;
    return followright (ff, getrootnode(fhandle));
  }

long nextrec (FLATHAND fhandle, long recno)
  {
    /* 1) if you have a right kid go to it and all the way to the left
       2) If you have no kids, go to your parent
          a) If you were the left kid he's next.
          b) if you were the right kid, follow parents until
             you are the left kid, if none, then end of list */
       
    sIndex k;
    FLATHAND ff;
    long lp;
    long par;
    int keepgoing = TRUE;

    lp = recno;

    if (lp == -1)
      return -1;

    if (fhandle == NULL)
      return -1;
    ff = fhandle->index;
    if (ff == NULL)
      return -1;

    if (flatread (ff, lp, (char *)&k) != 0)
      return -1;

    if (k.right != -1) /* simplest case (1) */
      return followleft(ff, k.right);

    /* it actually doesn't matter if we have a left kid, we never
       use it. So if we're here, just look for right side parents */
    while (keepgoing == TRUE)
      { 
        par = k.parent;
        if (par == -1)
          return -1; /* we're at the root trailing from the rightmost, no more */
        if (flatread (ff, par, (char *)&k) != 0)
          return -1;
        if (k.left == lp)
          return par; /* 2a. I was the left kid, parent is next */
        /* If here, then I'm the right kid, need to loop up */
        lp = par; /* 2b */
      }
    return 0;
  }
  
long prevrec (FLATHAND fhandle, long recno)
  {
    /* Same thing as nextrec, but backwards 
       1) if you have a left kid go to it and all the way to the right
       2) If you have no kids, go to your parent
          a) If you were the right kid he's next.
          b) if you were the left kid, follow parents until
             you are the right kid, if none, then end of list */
    sIndex k;
    FLATHAND ff;
    long lp;
    long par;
    int keepgoing = TRUE;
    
    lp = recno;

    if (lp == -1)
      return -1;

    if (fhandle == NULL)
      return -1;
    ff = fhandle->index;
    if (ff == NULL)
      return -1;

    if (flatread (ff, lp, (char *)&k) != 0)
      return -1;

    if (k.left != -1) /* simplest case (1) */
      return followright(ff, k.left);

    while (keepgoing == TRUE)
      { 
        par = k.parent;
        if (par == -1)
          return -1; /* we're at the root trailing from the rightmost, no more */
        if (flatread (ff, par, (char *)&k) != 0)
          return -1;
        if (k.right == lp)
          return par; /* 2a. I was the left kid, parent is next */
        /* If here, then I'm the right kid, need to loop up */
        lp = par; /* 2b */
      }
    return 0;
  }

/* Blob support (only works with indexes)
   The deal with blobs is that how efficient it is depends on your record
   size, so you may want to put your blobs in a separate file or sacrifice
   space on the end of non-blob records to make blobs more storage efficient.

   Blobs are stored as record after record after record key then data.
   Since we can have binary keys, we will put a B in front of the data
   and a long sequence number after it. Thus, keys will be 5 bytes longer
   than the index key length.
   The first 4 bytes of the blob will be it's size, not including the 4
   byte count.

*/

long blobsize (FLATHAND fhandle, char *key)
  { /* returns the size of the blob */
    /* returns -1 if not found. This can be used as test for existance
       because blobseek actually reads it in */

    char *bkey;
    long sz;
    long rec;
    char *buf;
    int ret;

    if (fhandle == NULL)
      return -1;
    if (fhandle->index == NULL) /* can't do blobs without index */
      return -1;

    bkey = getmem(fhandle->indexsize+7);
    buf  = getmem(fhandle->recordsize+5);

    bkey[0] = 'B';
    memcpy (bkey+1, key, fhandle->indexsize);
    rec = 0; /* find the first record */
    memcpy (bkey+1+fhandle->indexsize, &rec, 4); /* copy the long of 0 */

    ret = blob5seek (fhandle, bkey, buf);

    if (ret != 0)
      {
        freemem (buf);
        freemem (bkey);
        return -1;
      }

    /* the record starts at B + key + 4byte seq no. The first record
       starts with a long of the size */
    memcpy (&sz, buf+1+fhandle->indexsize+sizeof(long), sizeof(long));

    freemem (bkey);
    freemem (buf);
    return sz;
  }

int blobseek (FLATHAND fhandle, char *key, char *out)
  {
    /* returns: 0 = success, 1 = failed, 2 = not found */
    /* Failed could mean that it's there but corrupted */

    /* Caller must supply buffer */

    char *bkey;
    long sz;
    long recsize;
    long rec;
    char *buf;
    int ret;
    long outpos; /* position that we're writing in their buffer */
    int cpoffset;

    sz = blobsize (fhandle, key);
    if (sz == -1)
      return 1; /* failed */

    bkey = getmem(fhandle->indexsize+7);
    buf  = getmem(fhandle->recordsize+5);

    /* Start reading keys */
    bkey[0] = 'B';
    memcpy (bkey+1, key, fhandle->indexsize);

    outpos = 0;
    rec = 0; /* find the first record */

    while (rec != -1)
      {
        memcpy (bkey+1+fhandle->indexsize, &rec, sizeof(long)); /* copy the long of 0 */

        /* If we get to a record that SHOULD be there, but is not,
           it's a 'failed' */
        ret = blob5seek (fhandle, bkey, buf); /* extends the key by 5 chars */
        if (ret != 0)
          {
            freemem (buf);
            freemem (bkey);
            return -1;
          }
        /* See what we have to copy from this record */
        recsize = fhandle->recordsize;
        recsize -= 1 + fhandle->indexsize + sizeof(long); /* Total length of data */

        if (rec == 0) /* record 0 is special case, have to skip first 4 bytes */
          cpoffset = sizeof (long);
        else
          cpoffset = 0;

        if (recsize-cpoffset > sz) /* what's left to read */
          recsize = sz+cpoffset;

        memcpy (out+outpos, buf+1+fhandle->indexsize+sizeof(long)+cpoffset, recsize-cpoffset);

        sz -= recsize-cpoffset;
        outpos += recsize-cpoffset; /* Move through buffer of what to read into */

        if (sz == 0)
          {
            freemem (bkey);
            freemem (buf);
            return 0;
          }
        rec++;
      } /* while more records */
    return 0;
  }

int blobappend (FLATHAND fhandle, char *key, char *in, long size)
  {
    /* Returns 0 if successful, 1 if failed */
    /* Make record after record of data fitting in after-key space */
    /* If there's already a blob with this key, this is going to make
       a mess so we're going to be nice and delete one if it's already
       there. */
    /* Blobs are a bit different, the front of the data isn't the
       key, you have to pass a separate one. We need to sequence off
       the key */

    long sz;
    long recsize;
    long rec;
    char *buf;
    long inpos; /* position that we're reading in their buffer */
    int cpoffset;

    if (fhandle == NULL)
      return 1;
    if (fhandle->index == NULL) /* can't do blobs without index */
      return 1;

    sz = blobsize (fhandle, key);
    if (sz != -1)
      {
        /* Something's there, delete it */
        if (blobdelete (fhandle, key) != 0)
          return 1; /* something really bad happened */
      }

    buf = getmem(fhandle->recordsize+5);

    /* Make first key */
    buf[0] = 'B';
    memcpy (buf+1, key, fhandle->indexsize);

    inpos = 0;
    rec = 0; /* find the first record */

    while (rec != -1)
      {
        memcpy (buf+1+fhandle->indexsize, &rec, sizeof(long)); /* copy the long of the record for this key */
        
        /* See what we have to copy from this record */
        recsize = fhandle->recordsize;
        recsize -= 1 + fhandle->indexsize + sizeof(long); /* Total length of data */

        if (rec == 0) /* record 0 is special case, have to skip first 4 bytes */
          {
            cpoffset = sizeof (long);
            /* Write out the length to the buffer */
            memcpy (buf+1+fhandle->indexsize+sizeof(long), &size, sizeof(long));
          }
        else
          cpoffset = 0;

        if (recsize-cpoffset > size) /* what's left to write */
          recsize = size+cpoffset;

        memcpy (buf+1+fhandle->indexsize+sizeof(long)+cpoffset, in+inpos, recsize-cpoffset);

        /* Write it to disk. Now here's an interesting case. We have to
           make a extended key blob version of this too because while
           we don't care if flat knows about the key and data, it may
           pick a wrong best child while building the tree unless it
           knows about the last 4 chars of OUR key. */

        if (flat5append (fhandle, buf) != 0)
          { /* Something bad happened */
            freemem (buf);
            return 1;
          }

        inpos += recsize-cpoffset; /* Move through buffer of what to write */
        size -= recsize-cpoffset;
        if (size == 0)
          {
            freemem (buf);
            return 0;
          }
        rec++;
      } /* while more records */
    return 0;
  }

int blobdelete (FLATHAND fhandle, char *key)
  {
    /* Make keys and delete all records until you hit a hole */
    /* Due to the nature of blobs, being a hack, you have to
       read in a record to delete it, but blobs are supposed to
       be expensive anyway. */

    char *bkey;
    long sz;
    int ret;
    long rec;

    sz = blobsize (fhandle, key);
    if (sz == -1)
      return 1; /* failed */

    bkey = getmem(fhandle->indexsize+7);

    /* Start reading keys */
    bkey[0] = 'B';
    memcpy (bkey+1, key, fhandle->indexsize);

    rec = 0; /* find the first record */

    while (rec != -1)
      {
        memcpy (bkey+1+fhandle->indexsize, &rec, sizeof(long)); /* copy the long of 0 */

        /* If we get to a record that SHOULD be there, but is not,
           it's a 'failed' */
        ret = flat5seekdel (fhandle, bkey); /* extends the key by 5 chars */
        if (ret != 0) /* Did them all or error */
          {
            freemem (bkey);
            if (ret == 2)
              return 0; /* not found is good */
            return -1;
          }
        rec++;
      } /* while more records */
    return 0;
  }

/***************************************************************************/



void main6 (void)
  {
    /* The blob test */
    FLATHAND f;

    int ret;
    long sz;
    char b[500];
    char c[500];
    
    unlink ("test");
    unlink ("test.ind");
    
    f = flatopen ("test", 5, 20, TRUE, 5);

    
    strcpy (b, "1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz");

    ret = blobappend (f, "12345", b, strlen(b));

    dumpindex(f);
    dumpkey(f); 

    strcpy (b, "abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz");

    ret = blobappend (f, "zzzzz", b, strlen(b));
    dumpkey(f); 
    dumpindex(f);
printf ("\n");
    //problem is here...    
    blobdelete (f, "12345");
    dumpindex(f);
    dumpkey(f);

    ret = blobappend (f, "12345", b, strlen(b));
    sz = blobsize(f, "12345");
    dumpkey(f); 

    ret = blobseek(f, "zzzzz", c);
printf ("[%s]\n", c);
printf ("[abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz]\n");
    flatclose (&f);
  }

/* for testing */

void main3 (void)
  {
    FLATHAND f;
#define CON 500

    int a[CON];

    int lp;
    int done = FALSE;
    
    unlink ("test");
    unlink ("test.ind");
    
    f = flatopen ("test", 5, 20, TRUE, 5);

    for (lp = 0; lp < CON; lp++)
      a[lp] = 0;
    done = FALSE;
    
    for (lp = 0; lp < CON; lp++)
      {
        int v;
        v = rand() % CON;
        if (a[v] == FALSE)
          {
            char r[40];
            sprintf (r, "%05dfafdsfsd", v);
            flatappend (f, r);
            a[v] = TRUE;
          }
      }
    /* Now fill in all the holes */
    for (lp = 0; lp < CON; lp++)
      {
        int v;
        v = lp;
        if (a[v] == FALSE)
          {
            char r[40];
            sprintf (r, "%05dzvcxzvrg", v);
            flatappend (f, r);
            a[v] = TRUE;
          }
      }
    
    dumpkey(f); 
    printf ("The Ordered index:\n");
    dumpindex(f); 

    for (lp = 0; lp < ((CON*8)/10); lp++)
      {
        flatreaddel (f, 0);
      }

    printf ("After delete 8000 from rec 0:\n");
    dumpkey(f); 
    printf ("The deleted index:\n");
    dumpindex(f); 

    reindex(f);
    printf ("After reindexing\n");
    dumpkey(f); 
    printf ("The deleted index:\n");
    dumpindex(f); 
    

    flatclose (&f);

  }

void testmain (void)
  {
    /* small blobs */
    FLATHAND f;

    int ret;
    long sz;
    char *b;
    
    unlink ("test");
    unlink ("test.ind");
    
    f = flatopen ("test", 5, 1024, TRUE, 5);
    
    b = getmem(1000005);

    strcpy (b, "1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz1234567890abcdefghijklmnopqrstuvwxyz");

printf ("About to append.\n");
    ret = blobappend (f, "10000", b, 1000005);
printf ("Done.\n");

printf ("about to read.\n");
    sz = blobsize (f, "10000");
    ret = blobseek (f, "10000", b);
printf ("Done.\n");
scanf("%s", b);
dumpindex(f);

    flatclose (&f);
  }
