/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef INCL_FLAT_H
#define INCL_FLAT_H

typedef struct tagflathand *FLATHAND;

struct tagflathand
  {
    int f;
    int headersize;
    int recordsize;
    FLATHAND index;
    int indexsize;
  };

typedef struct tagindexstruct
  {
    /* Layout of the index file */
    /* Record numbers..., -1 is no record this way */
    long parent;
    long left;
    long right;
  } sIndex;

typedef struct tagheaderstruct
  {
    /* Layout of the header of the index file */
    long rootnode;
  } sheaderIndex;


/* Start of API */

/* All in's and out's are from the function's point of view */
/* Keys and data can be anything including nulls.
   Key matches exact length of index key. There is no softseeking */

/* open close routines */
FLATHAND flatopen (char *file, int headersize, int recordsize, int index, int keysize);
void flatclose (FLATHAND *f);

/* work with header */
int getheader (FLATHAND fhandle, char *out);
int setheader (FLATHAND fhandle, char *in);

/* Direct file reads/writes */
int flatread (FLATHAND fhandle, long recno, char *out);
/* This will corrupt the index if you're using one, so don't use it */
int flatwrite (FLATHAND fhandle, long recno, char *in);
long flatrecno (FLATHAND fhandle);

/* Key based reads/writes */
int flatseek (FLATHAND fhandle, char *key, int keylen, char *out);
int flatupdate (FLATHAND fhandle, char *key, char *in);
int flatappend (FLATHAND fhandle, char *in);

/* Record removal */
int flatreaddel (FLATHAND fhandle, long recno);
int flatseekdel (FLATHAND fhandle, char *key, int keylen);

/* Index support */
long firstrec (FLATHAND fhandle);
long flatseekrec (FLATHAND fhandle, char *key, int keylen); /* returns record number or -1 if not found */
long lastrec (FLATHAND fhandle);
long nextrec (FLATHAND fhandle, long recno);
long prevrec (FLATHAND fhandle, long recno);

/* Blob support (only works with indexes) */
long blobsize  (FLATHAND fhandle, char *key); /* returns the size of the blob */
int blobseek   (FLATHAND fhandle, char *key, char *out);
int blobappend (FLATHAND fhandle, char *key, char *in, long size);
int blobdelete (FLATHAND fhandle, char *key);

void reindex(FLATHAND fhandle);

/* End of API */


#endif
