/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef FSERVER_H
#define FSERVER_H

#include <flat/flat.h>
#include <flat/fpack.h>
#include <Q/entry.h>
#include <Q/zary.h>

int main(int num, char *opts[]);
int runner (int num, ConfigCraft &ar, Acceptor &ac);

class ConfigCraft;
class NoteCraft;
class Acceptor;
class FFilePtr;

class FlatServer
  {
    public:
      FlatServer(ConfigCraft &Ar, Acceptor &Ac, NoteCraft &Ls, int num);
      ~FlatServer(void) {}
      int run(void);
      int acceptconnection(int acceptsocket);
      int process_request(void);
      void startsession(void);
      
      fresp getfp(socketio &sco, FlatPacket &fp);
      fresp sendfr(socketio &sco, FlatResponse &fr);

      int check_connection(void);
      int closeconn(void);
      fresp openfile(socketio &sco, FlatResponse &fr);
      fresp closefile(socketio &sco, FlatPacket &fp);
      void szfix(Memory &m, UINT size);
 
      void FSgetheader(socketio &sco, FlatPacket &fp);
      void FSsetheader(socketio &sco, FlatPacket &fp);
      void FSreadrec(socketio &sco, FlatPacket &fp);
      void FSwriterec(socketio &sco, FlatPacket &fp);
      void FSdeleterec(socketio &sco, FlatPacket &fp);
      void FSseekrec(socketio &sco, FlatPacket &fp);
      void FSupdate(socketio &sco, FlatPacket &fp);
      void FSatomicupdate(socketio &sco, FlatPacket &fp);
      void FSinsert(socketio &sco, FlatPacket &fp);
      void FSatomicinsert(socketio &sco, FlatPacket &fp);
      void FSdelete(socketio &sco, FlatPacket &fp);
      void FSselect(socketio &sco, FlatPacket &fp);
      void FSfirstrec(socketio &sco, FlatPacket &fp);
      void FSlastrec(socketio &sco, FlatPacket &fp);
      void FSseekrecno(socketio &sco, FlatPacket &fp);
      void FSnextrec(socketio &sco, FlatPacket &fp);
      void FSprevrec(socketio &sco, FlatPacket &fp);
      void FSBseek(socketio &sco, FlatPacket &fp);
      void FSBinsert(socketio &sco, FlatPacket &fp);
      void FSBatomicinsert(socketio &sco, FlatPacket &fp);
      void FSBdelete(socketio &sco, FlatPacket &fp);
      void FSreindex(socketio &sco, FlatPacket &fp);
            
    private:
      FLATHAND getfile(FlatPacket &fp, FFilePtr *&sffp);

      ConfigCraft &ar;
      NoteCraft &ls;
      Acceptor &ac;
      socketio *sco;
      
      zary fileary; // the array of fileptrs
      int tmout;
      int highsocket;
  };

class FFile
  {
    public:
      FFile(char *fn, FLATHAND sfh, ULONG sheadersize, ULONG srecordsize,  
                                    ULONG sindex, ULONG skeysize)
        {
          fh = sfh;
          refcount = 0;
          filename = fn; // so we can compare on future opens
          headersize = sheadersize;
          recordsize = srecordsize;
          index = sindex;
          keysize = skeysize;
          /* here we make the mutex for this file. You can defeat this 
             mechanism by defeating ftok to get a different id at the
             same file. Though I'm not sure how you'd go about doing that.
             This mutex id will be the same across all kid processes, so
             when we lock on write for this file, no other process can
             write to this file at the same time. That's the plan, anyway. */
           
          int mutexfileid = ftok(fn, 'A');
          if (mutexfileid == -1)
            fh = NULL; // denote bad, caller will close this shortly
          rwmutex = new RWMutexLock(mutexfileid); 
        }
      ~FFile(void) 
        { 
          flatclose(&fh); 
          // 11/5/02 and here's where we free the mutex
          delete rwmutex;
        }
      void inc(void) { refcount++; }
      void dec(void) { refcount--; }
      int count(void) { return refcount; }
      char *getfname(void) { return filename; }
      FLATHAND getfh(void) { return fh; }
      ULONG getheadersize(void) { return headersize; }
      ULONG getrecordsize(void) { return recordsize; }
      ULONG getindex(void) { return index; }
      ULONG getkeysize(void) { return keysize; }
      RWMutexLock &getlock(void) { return *rwmutex; }
      
    private:
      int refcount;
      Memory filename;
      FLATHAND fh; // this is a ptr
      ULONG headersize, recordsize, index, keysize;
      RWMutexLock *rwmutex;

  };

class FFilePtr : public entry
  {
    // almost but not quite a smart pointer
    // as we put these in the list, the refcount will go up.
    // as they are deleted, it will dec and when there are none it will go away
    public:
      FFilePtr (FFile *ff, SOCKET s)
        {
          ptr = ff;
          ptr->inc();
          fc = s;
        }
      virtual ~FFilePtr (void)
        {
          ptr->dec();
          if (ptr->count() == 0)
            delete ptr;
        }
      SOCKET getfc(void) { return fc; }
      FFile *getffile(void) { return ptr; }
      FLATHAND getfh(void) { return ptr->getfh(); }
      ULONG getheadersize(void) { return ptr->getheadersize(); }
      ULONG getrecordsize(void) { return ptr->getrecordsize(); }
      ULONG getindex(void) { return ptr->getindex(); }
      ULONG getkeysize(void) { return ptr->getkeysize(); }
      RWMutexLock &getlock(void) { return ptr->getlock(); }
      
    private:
      FFile *ptr;
      SOCKET fc;
  };

#endif


