/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DBH
#define DBH

/****************  The user directory file.  ****************/
/* This table is tall and skinny, just has the username and
   defstore and userid, so we can go find their mailbox. This
   is the readonly table that will get synced if changed in
   multiple places. */
   
#define TXT_DIRECTORYFILE "directory.dat"
#define TXT_DIRECTORYFILEVERSION 1

typedef struct
  {
    UINT version;
    UINT highestid; // sequencer for user id
    char filler[16];
  } dirheader;
  
typedef struct
  { 
    char username[SZ_USERNAME]; // key, always all lower case, padded with spaces
    char domain[SZ_DOMAIN]; // padded with spaces, also part of key
    UINT userid; // system wide unique id
    char defstore[SZ_DEFSTORE];  // default storage location anchor where the user is located 
    char filler[400];
  } dirrec;

/****************  The user file.  ****************/

#define TXT_USERFILE "user.dat"
#define TXT_USERFILEVERSION 1

typedef struct
  {
    UINT version;
    char filler[16];
  } userheader;
  
typedef struct
  { 
    UINT userid; // system wide unique i, name map in directory
    UINT status; // active, disabled...
    UINT type; // alias, run program...
    UINT access; // access level
    char password[SZ_PASSWORD];
    char alt[SZ_USERNAME+SZ_DOMAIN]; // where it aliases to, in same domain or program to run or smtpserver to hit
    UINT quotamax;
    UINT quotaused;
    UINT allowsmtp; // for webmail, 0x00 or 0x01 should be allowed to send mail from webmail
    
    char filler[400];
  } userrec;

/****************  The user folder file.  ***************/

#define TXT_FOLDERFILE "folder.dat"
#define TXT_FOLDERFILEVERSION 2
// version 2, change inbox name from inbox to /inbox

typedef struct
  {
    UINT version;
    UINT highestid;
    char filler[16];
  } folderheader;
  
typedef struct
  { 
    UINT userid; // part of key
    char foldernamekey[SZ_FOLDERNAME]; // part of key, all lower case space padded
    char foldername[SZ_FOLDERNAME]; // printable version of foldername
    UINT folderid;

    char filler[100];
  } folderrec;

/****************  the message pointer file.  ***********/

#define TXT_MSGPTRFILE "msgptr.dat"
#define TXT_MSGPTRFILEVERSION 1

typedef struct
  {
    UINT version;
    char filler[16];
  } msgptrheader;
  
typedef struct
  { 
    UINT userid; // part of key
    UINT folderid; // part of key
    char msgid[SZ_MSGID]; // part of key padded with space
    UINT status; // bitfields, read, deleted, invisible, flagged, tmpdeleted (pop3)
    UINT insertdate;
    UINT msgsize; 
    char defstore[SZ_DEFSTORE]; /* where this message is located. */
    
    char filler[100];
  } msgptrrec;

/****************  the resource file.  ***********/

/* Generic key-value table for holding system settings
   like the accept domain list. */
   
#define TXT_RESOURCEFILE "resource.dat"
#define TXT_RESOURCEFILEVERSION 1

typedef struct
  {
    UINT version;
    char filler[4096];
  } resourceheader;
  
typedef struct
  {
    char key[SZ_RESKEY];
    char value[SZ_RESVAL];
     
    char filler[512];
  } resourcerec;

/**********************************************************/


#endif
