/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <Q/memm.h>
#include <Q/socketio.h>
#include <Q/zlist.h>
#include <Q/connector.h>
#include <Q/acceptor.h>
#include <Q/configcraft.h>
#include <Q/notecraft.h>
#include <Q/entry.h>
#include <flat/fc.h>

#include "global.h"
#include "io.h"
#include "utils.h"
#include "dbfinder.h"

/* The outside world doesn't really have to know about 
   Flatdatabases, we get the connect string, look up the server
   and port, make a connection, and return the FlatConnection. */

dbfinder::dbfinder (ConfigCraft &Ar, NoteCraft &Ls, char *primedefstore) : ar(Ar), ls(Ls)
  {
    // look up primedefstore, connect to it, add it to the list.

    // if this returns null, we'll know when we try to use it.    
    connentry *cn = addnewconnection(primedefstore);
    prime = cn; // we just happen to have a copy handy
  }

FlatConnection *dbfinder::getconn(char *defstore)
  { // return or create and return connection for this defstore 
  
    int ret = connlist.FindPosByString(defstore);
    if (ret != -1)
      {
        connentry *ce = (connentry *)connlist.GetItem(ret);
        return ce->getfc();
      }
    // have to make the connection, and add it to list.
    connentry *cn = addnewconnection(defstore);
    if (cn == NULL)
      return NULL;
    return cn->getfc();
  }

connentry *dbfinder::addnewconnection(char *defstore)
  {
    /* build a connection add it to the list, and return it */
    /* Return null if it doesn't work. */
    Memory server;
    Memory junk;
    int port;
    int timeout = val(ar[TXT_SOCKETTIMEOUT]);

    Memory ds = ar[defstore];
    if (strlen(ds) == 0)
      {
        ls (NOTEERR) << "Can't add new connection, defstore " << defstore << " has no value." << endl;
        return NULL; // uh oh.
      }
    
    splithostportpath(ds, server, port, junk);
    FlatDatabase *db = new FlatDatabase(server, port, timeout);
    FlatConnection *conn = new FlatConnection();
    *conn = db->Connect();
    if (conn->getsco() == NULL)
      {
        ls (NOTEERR) << "Unable to connect to defstore: " << (char *)server << ":" << port << endl;
        delete conn;
        delete db;
        return NULL;
      }
    connentry *cn = new connentry(db, conn, defstore, ar, ls);
    connlist.Add(cn); // the list owns the connection
    return cn;
  }



