/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DBFINDER_H
#define DBFINDER_H


/* We need something to conviently and unobtrusivly get a database
   connection for a given defstore.
   Keep in mind the defstore for mailboxes and defstore for message
   bodies are two completely different things.
   defstore for messages are handled in msgstore.*
   This is just for mailbox finding. */
   
/* It will get the first connection to make then look up the others
   in the config file and make connections on demand as they
   are needed. When somebody looks one up, it will return the
   appropriate db connection. */
   
// the instance of connentry actually owns the flatconnection
class connentry : public entry
  {
    public:
      connentry (FlatDatabase *Dbc, FlatConnection *Fc, char *Defstore, ConfigCraft &ar, NoteCraft &ls)
        { 
          dbc = Dbc; 
          fc = Fc; 
          defstore = Defstore; 
          /* By opening all the files here, flat will refcount and not
             keep opening and closing the file since we have at least one */
          trf = new tresource(*fc, ar, ls);
          tuf = new tuser(*fc, ar, ls);
          tff = new tfolder(*fc, ar, ls);
          tmf = new tmsgptr(*fc, ar, ls);
          
          trf->checkversion();
          tuf->checkversion();
          tff->checkversion();
          tmf->checkversion();
        }
      ~connentry(void) 
        { 
          delete tmf; // close the database files
          delete tff;
          delete tuf;
          delete trf;
          delete fc; // drop the connection
          // we don't delete dbc, we don't own the database, just
          // the connection.
        } // this will effectivly drop the connection to the database
      
      FlatConnection *getfc(void) { return fc; }
      FlatDatabase *getdbc(void) { return dbc; }
      char *getdefstore(void) { return defstore; }
      char *Finder(void) { return defstore; }

    private:
      FlatDatabase *dbc;
      FlatConnection *fc;
      Memory defstore;
      tresource *trf;
      tuser *tuf;
      tfolder *tff;
      tmsgptr *tmf;
  };
   
class dbfinder
  {
    public:
      dbfinder (ConfigCraft &Ar, NoteCraft &Ls, char *primedefstore);
      ~dbfinder(void) {}
      FlatConnection *getprimeconn(void) 
        { 
          if (prime == NULL)
            return NULL;
          else
            return prime->getfc(); 
        }
      FlatConnection *getconn(char *defstore); // return or create and return connection for this defstore 
      
    private:
      connentry *addnewconnection(char *defstore);
    
      connentry *prime; // the main directory lookup connection
      zlist connlist;
      ConfigCraft &ar;
      NoteCraft &ls;
  };

#endif











