/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef GLOBALH
#define GLOBALH

#include "release.h"

#define UCHAR unsigned char
#define USHORT unsigned short

#define TXT_AFFILEVERSION 1 // af fileformat version number

#define SZ_DEFSTORE 6 // size of storage anchor string both for user and message store

#define SZ_USERNAME 50
#define SZ_PASSWORD 50
#define SZ_DOMAIN 80

#define SZ_FOLDERNAME 200 // max length of nested folder names save these/kid1/day1
#define SZ_MSGID 100 // systemwide unique message id

/* Resource table key value sizes */
#define SZ_RESKEY 100
#define SZ_RESVAL 1024

/* Function return codes */
#define RET_OK 1
#define RET_NOTFOUND 2
#define RET_ERROR 3
#define RET_NEEDDATA 4 // for msgstore::store cmd
#define RET_CONNECTFAILED 5
#define RET_ENDOFDATA 6
#define RET_PERMANENT_ERROR 7
#define RET_DUPLICATE 8
#define RET_MSGSTORE_ERROR 9
#define RET_PARTIAL 10

#define O3NAME "o3" // name of global config file

#define MAX_ALIAS_DEPTH 5 // number of follows in directory for aliases

// definitions for record type in user table
#define USERTYPE_MAIL 1
#define USERTYPE_ALIAS 2
#define USERTYPE_PROGRAM 3
#define USERTYPE_FILE 4
#define USERTYPE_SMTP 5

// definitions for status in user table
#define USERSTATUS_ENABLED 1
#define USERSTATUS_DISABLED 2

// definitions for allowsmtp in user table
#define ALLOWSMTP_YES 1
#define ALLOWSMTP_NO 2

/* global system mutex id's, come up with something better for this, eh?
   This system presumes that any changes atomicized by these mutexes occur on
   the same machine. They are mostly admin functions anyway. */
#define MUTEX_DIRADD      1023241
#define MUTEX_FOLDERADD   1023242
#define MUTEX_MSGPTRADD   1023243
#define MUTEX_RESOURCEADD 1023244

#define TRANSFER_BUF 102400 // 100K at a time

#define CREATEMODE S_IRWXU // everything about this system is RWX owner only
#define TXT_FOPENREAD "r" // for example, in windows this would be "rb"
#define TXT_FOPENWRITE "w"
#define TXT_FOPENAPPEND "a"

#define TXT_INBOX "/Inbox"  // what the system inbox is called

#define TXT_DEFAULTHOSTNAME "localhost"
#define TXT_LOCKPREFIX "working."
#define MIN_QUEUE_RETRY_SECONDS  (15*60) // 15 minutes

/* for pop */
#define TXT_DEFAULT_DOMAIN "default-domain"
#define TXT_BAD_SLEEP_SECONDS "bad-sleep-seconds"
#define TXT_BAD_AUTH_MAX "bad-auth-max"

/* Messageptr status bit flag numbers and ids */

#define MAX_FLAGS          32

#define MSGFLAG_VISIBLE    1
#define MSGFLAG_READ       3
#define MSGFLAG_FLAGGED    4
#define MSGFLAG_TMPDEL     5
#define MSGFLAG_REPLIED    6
#define MSGFLAG_FORWARDED  7

#define TXT_MSGFLAG_VISIBLE    "visible"
#define TXT_MSGFLAG_READ       "read"
#define TXT_MSGFLAG_FLAGGED    "flagged"
#define TXT_MSGFLAG_TMPDEL     "deleted"
#define TXT_MSGFLAG_REPLIED    "replied"
#define TXT_MSGFLAG_FORWARDED  "forwarded"

#define MSGSTATUS_VISIBLE   (1 << (MSGFLAG_VISIBLE   - 1)) // 1 // I'm here, but you can't SEE me :-)
#define MSGSTATUS_READ      (1 << (MSGFLAG_READ      - 1)) // 4
#define MSGSTATUS_FLAGGED   (1 << (MSGFLAG_FLAGGED   - 1)) // 8
#define MSGSTATUS_TMPDEL    (1 << (MSGFLAG_TMPDEL    - 1)) // 16
#define MSGSTATUS_REPLIED   (1 << (MSGFLAG_REPLIED   - 1)) // 32
#define MSGSTATUS_FORWARDED (1 << (MSGFLAG_FORWARDED - 1)) // 64

/* Useful mail header field names. */

#define TXT_TO                        "To:"
#define TXT_FROM                      "From:"
#define TXT_REPLY_TO                  "Reply-To:" 
#define TXT_MESSAGEID                 "Message-ID:"
#define TXT_SUBJECT                   "Subject:"
#define TXT_MIME_VERSION              "MIME-Version:"
#define TXT_CONTENT_TYPE              "Content-Type:"
#define TXT_CONTENT_TRANSFER_ENCODING "Content-Transfer-Encoding:"
#define TXT_CONTENT_DISPOSITION       "Content-Disposition:"
#define TXT_DATE                      "Date:"

#define TXT_7BIT                      "7bit"
#define TXT_TEXT_PLAIN                "text/plain"
#define TXT_MIMETEXTMSG               "This is a multi-part message in MIME format."
#define TXT_CHARSET                   "charset=us-ascii"
#define TXT_MULTIPART_MIXED           "multipart/mixed"
#define TXT_MULTIPART                 "multipart"
#define TXT_INLINE                    "inline"
#define TXT_BOUNDARY                  "boundary"
#define TXT_MESSAGE_RFC822            "message/rfc822"
#define TXT_ATTACHMENT                "attachment"

#define RFCLINESIZE 78 
// speaking of RFC
#define TXT_MAIL_FROM                 "Mail From:"
#define TXT_SIZE                      "SIZE"

/*                      This stuff is in the config file                */

#define TXT_PRIME_DEFSTORE "prime-defstore" // the first connection to make
#define TXT_FLUSH_OFTEN "flush-often" // for the message store

// the config option name of the root msg store directory
#define TXT_MSGROOTDIR "msgrootdirectory"

// the config option name of the root flatserver data directory
#define TXT_DBROOTDIR "dbrootdirectory"

/* for smtp domain names and the message defstore */
#define TXT_CACHE_WIPE_INTERVAL "cache-wipe-interval"

/* how often we should try and reprocess temporary failed queued items */
#define TXT_QUEUE_RETRY_SECONDS "queue-retry-seconds"
#define TXT_MAX_DELIVERY_ATTEMPTS "max-delivery-attempts"

#define TXT_GLOBAL_OUTBOUND_SMTP_SERVER "global-outbound-smtp-server"
#define TXT_GLOBAL_OUTBOUND_SMTP_PORT   "global-outbound-smtp-port"

/* Whom bounce messages are from */
#define TXT_SYSTEMMAILFROM "system-mailfrom"

/* The size we will tell clients is the maximum message accept 
   size (RFC 1870), 0 = no limit. */
#define TXT_ESMTP_SIZE "esmtp-size"




/*                      These are in the resource table                  */

#define TXT_MESSAGE_DEFSTORE "message-defstore"

#define TXT_DOMAINPREFIX "domain-" // heh heh, must be lower case, keys in resource table are lower case
#define TXT_RELAYDOMAINPREFIX "relaydomain-" 





#endif


