/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef IOH
#define IOH

extern Memory DBROOTDIR;

/* Must call this before you do anything to the mail system's io */
void ioinit(ConfigCraft &ar, NoteCraft &ls);

int frtoret(fresp ret);
void getdbrootdir(Memory &rd); // one global place for all apps to get this

/* I have this rule about not including headers from headers but
   this is a useful exception. */
#include "db.h"

// circular reference, io and dbfinder use each other
class dbfinder;

class tdirectory
  {
    public:
      tdirectory(dbfinder &dbf, ConfigCraft &ar, NoteCraft &ls);
      ~tdirectory(void) {}
      
      inline int getkeysize(void);
      int lookup(char *name, char *domain);
      int lookupalias(char *Name, char *Domain);
      int add(char *name, char *domain); 
      int del(char *name, char *domain);
      int update(char *name, char *domain);
      int getuserlist(zlist &list);
      void checkversion();

      /* While this seems bad and very unobject oriented, I don't see the
         need to make a get_ set_ and sizeof_ for every single field in 
         every database. */
         
      dirrec &getdata(void) { return data; }
      int reindex(void) { return frtoret(fdirectory.reindex()); } 
      
    private:
      void makedirectorykey(Memory &key, char *Name, char *Domain, int &keysz);

      dbfinder &finder;
      FlatConnection &dbc;
      FlatFile fdirectory;
      ConfigCraft &ar;
      NoteCraft &ls;
      dirrec data;
      Memory fn;
  };

class tuser
  {
    public:
      tuser(FlatConnection &f, ConfigCraft &ar, NoteCraft &ls);
      ~tuser(void) {}
      
      inline int getkeysize(void);
      int lookup(UINT userid);
      int add(UINT userid); 
      int del(UINT userid);
      int update(UINT userid);
      userrec &getdata(void) { return data; }
      int reindex(void) { return frtoret(fuser.reindex()); } 
      void checkversion();
      
    private:
      void makeuserkey(Memory &key, UINT userid, int &keysz);

      FlatConnection &dbc;
      FlatFile fuser;
      ConfigCraft &ar;
      NoteCraft &ls;
      userrec data;
      Memory fn;
  };
  
class tfolder
  {
    public:
      tfolder(FlatConnection &f, ConfigCraft &ar, NoteCraft &ls);
      ~tfolder(void) {}
      
      inline int getkeysize(void);
      int lookup(UINT userid, char *Foldername, int exact = TRUE);
      int add(UINT userid, char *Foldername); 
      int del(UINT userid, char *Foldername);
      int update(UINT userid, char *Foldername);
      folderrec &getdata(void) { return data; }
      int deluser(UINT userid);
      int reindex(void) { return frtoret(ffolder.reindex()); } 
      void checkversion();
      int getfolderlist(UINT userid, zlist &list);
      
    private:
      void makefolderkey(Memory &key, UINT userid, char *Foldername, int &keysz);

      FlatConnection &dbc;
      FlatFile ffolder;
      ConfigCraft &ar;
      NoteCraft &ls;
      folderrec data;
      Memory fn;
  };
  
class tmsgptr
  {
    public:
      tmsgptr(FlatConnection &f, ConfigCraft &ar, NoteCraft &ls);
      ~tmsgptr(void) {}
      
      inline int getkeysize(void);
      int lookup(UINT userid, UINT folderid, char *msgid, int exact = TRUE);
      int lookupuserfolder(UINT userid, UINT folderid, zlist &list);
      int add(UINT userid, UINT folderid, char *msgid); 
      int del(UINT userid, UINT folderid, char *msgid, int doalsofile);
      int update(UINT userid, UINT folderid, char *msgid);
      msgptrrec &getdata(void) { return data; }
      int deluser(UINT userid);
      int deluserfolder(UINT userid, UINT folderid);
      // for recalc, a passthrough to the flatlayer.
      fresp iselect(Memory &key, zlist &list, int keysize) 
        { return fmsgptr.iselect(key, list, keysize); } // cheap nasty stuff, that.
      int reindex(void) { return frtoret(fmsgptr.reindex()); } 
      void checkversion();

    private:
      void makemsgptrkey(Memory &key, UINT userid, UINT folderid, char *msgid, int &keysz);

      FlatConnection &dbc;
      FlatFile fmsgptr;
      ConfigCraft &ar;
      NoteCraft &ls;
      msgptrrec data;
      Memory fn;
  };

class tresource
  {
    public:
      tresource(FlatConnection &f, ConfigCraft &ar, NoteCraft &ls);
      ~tresource(void) {}
      
      inline int getkeysize(void);
      int lookup(char *rkey);
      int add(char *rkey);
      int del(char *rkey);
      int update(char *rkey);
      int getdomainlist(zlist &list);
      resourcerec &getdata(void) { return data; }
      int reindex(void) { return frtoret(fresource.reindex()); } 
      void checkversion();
      
    private:
      void makeresourcekey(Memory &key, char *rkey, int &keysz);

      FlatConnection &dbc;
      FlatFile fresource;
      ConfigCraft &ar;
      NoteCraft &ls;
      resourcerec data;
      Memory fn;
  };
  



#endif


