/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef MSGSTOREH
#define MSGSTOREH


/* This is the message store for the O3 mail system.
   Relativly standalone. You give it a world unique key
   for the message, and a 6 character defstore
   for scaling and it will store the message or retrieve the
   message or bits of message for you.
   
   This implementation will store the message as a file
   reference counted in the filename.
   When you go to store the message, you say 
   'store this' it will say, okay done (as in it already
   exists, and it just upped the ref count) or
   okay, send data, and it will write to a file then
   you close it.
   
   In this implementation only the first character of the
   defstore will be used, it will be the number of depth
   levels of directory to write out from the filename. 
   But all 6 characters are for ID. */
   

class msgstore
  {
    public:
      msgstore::msgstore(ConfigCraft &ar, NoteCraft &Ls, char *Defstore, char *Msgid, int flushoften);
      ~msgstore(void);
      
      int store(void); /* tells you if you need to send data */
      int remove(void); /* drops ref count, deletes if at 0 */
      int getsize(UINT &size);
      int seek(UINT pos); // from beginning
      int read(char *outbuf, int amount, int &got);
      int write(char *inbuf, int amount); // append
      int close(void);
      int readline(Memory &mline);

    private:
      void getrootdir(Memory &rd, int checkdir = FALSE);
      void getfn(Memory &fn, int checkdir = FALSE);
      int getrefcount(Memory &fn, Memory *rootfn = NULL);
      void statbuild(char *fn);
      int increfcount(Memory &fn, int *newref);
      int decrefcount(Memory &fn, int *newref);
  
      NoteCraft &ls;
      Memory defstore;
      Memory msgid;
      Memory rootdir;
      int reading;
      FILE *f;
      int forceflush;
      int badfile;
      Memory badfilename;
  };



#endif



