/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef POP_H
#define POP_H

#define TXT_USER "USER"
#define TXT_PASS "PASS"
#define TXT_APOP "APOP"
#define TXT_RSET "RSET"
#define TXT_STAT "STAT"
#define TXT_LIST "LIST"
#define TXT_RETR "RETR"
#define TXT_DELE "DELE"
#define TXT_TOP  "TOP"
#define TXT_UIDL "UIDL"
#define TXT_QUIT "QUIT"
#define TXT_LAST "LAST"
#define TXT_NOOP "NOOP"
#define TXT_HELP "HELP"
#define TXT_CAPA "CAPA"

#define TXT_LFLD "LFLD"
#define TXT_SFLD "SFLD"
#define TXT_CFLD "CFLD"
#define TXT_DFLD "DFLD"
#define TXT_PERS "PERS" // persistent connection mode (doesn't time out)

#define TXT_MOVE "MOVE"
#define TXT_COPY "COPY"

#define TXT_SETF "SETF"
#define TXT_GETF "GETF"

#define TXT_LPRT "LPRT"

#define TXT_QUOT "QUOT"

int main(int num, char *opts[]);
void addconfigdefaults(ConfigCraft &ar);
int runner(int num, ConfigCraft &ar, Acceptor &ac);
int insertdatesort (const void *i1, const void *i2);

class pop
  {
    public:
      typedef enum 
        {
          needauth = 0,
          needpass = 1,
          neednothing = 2
        } statetype;
        
      pop (ConfigCraft &Ar, Acceptor &Ac, NoteCraft &Ls, int num);
      ~pop(void);
      int run(void);

    private:
      void startsession(void);
      void response(char *s);
      char *expect(void);
      void heading(void);
      void userdomaincheck(void);
      void processuser(Memory &l);
      UINT lookupuser(int &bail);
      int processpass(Memory &l);
      void loadmailbox(char *foldername);
      int processapop(Memory &l);
      void processreset(void);
      void processlist(Memory &l);
      void processuidl(Memory &l);
      void processstat(void);
      void msginfo(Memory &l, int isuidl);
      void processretr(Memory &l);
      void processtop(Memory &l);
      void goretr(Memory &l, int istop);
      void processdele(Memory &l);
      void statusflush(void);
      void processquit(void);
      void adjustquota(UINT sz);
      void processlast(void);
      void processhelp(void);
      void processcapa(void);
      int getmsgpos(char *p);
      void processpers(void);

      /* pop 4, folder functions */
      void processsfld(Memory &l);
      void processlfld(Memory &l);
      void processcfld(Memory &l);
      void processdfld(Memory &l);
      int processmove(Memory &l);
      int processcopy(Memory &l);
      int getfolderparam(Memory &name, Memory in);
      int getfolderpiece(Memory &leader, char *&p);
      int validatefolder(zlist &pieces, Memory &fname);
      int processmovecopy(Memory &fld, int iscopy);

      // flags
      void processsetf(Memory &l);
      void processgetf(Memory &l);

      // quota
      void processquot(Memory &l);

      // partial message retrieval
      void processlprt(Memory &l);
      int partlist(int &nextret, Memory &readbuffer, msgstore &ms, Memory prefix, Memory boundary, int insubmessage, Memory &returnlastline, char *findpartname = NULL, int lines = -1);
      void parsecontenttype(Memory &ct, Memory &boundary, int &multipart, int &insubmessage);
      int unfoldreadline(int &nextret, Memory &buffer, msgstore &ms, Memory &l, int inheader, Memory &realline);
      void foldout(char *respin);
      void processretrpart(int istop, msgstore &ms, int mlines, char *partname);
      void showthisline(char *prefix, int partcount, char *findpartname, char *l, int lines, int inheader);
      
      ConfigCraft &ar;
      Acceptor &ac;
      NoteCraft &ls;
      dbfinder *finder;
      tdirectory *dirholder;
      
      socketio *sco;
      int state;
      Memory xmsg; // temporary persistent message returner
      
      int persistent; // time out or not.
      Memory xhostname;
      Memory apopheading;
      Memory username;
      Memory domain;
      tuser *user; // not null when we get it.
      Memory currentfoldername; // current foldername "/inbox" stored in pop4 protocol format, without trailing hierarchy delim
      tfolder *folder; // current session folder
      FlatConnection *mailstore; // when we look up the user this gets set
      int badauth;
      zlist msglist; // list of msgpacks
      UINT *originalstat; // list of original status from folder load
      hashtable<int> *uidlpos; // hashtable to find message position by uidl

      /* Support for the mime parser. These would be in the mime parser class
         if there was one :-) */
      int foundpart; // a sort of global flag (erm, hack)
      int linesout;
  };



#endif

