/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef SMTP_H
#define SMTP_H

#define TXT_QUEUEDIRECTORY        "queuedirectory"
#define TXT_QUEUEDATADIRECTORY    "queuedatadirectory"
#define TXT_DEADQUEUEDIRECTORY    "deadqueuedirectory"

#define TXT_ALLOW_RELAY_INTERFACE "allow-relay-interface"
#define TXT_ALLOW_RELAY_HOSTS     "allow-relay-hosts"


#define TXT_HELO "HELO"
#define TXT_EHLO "EHLO"
#define TXT_MAIL "MAIL FROM"
#define TXT_RCPT "RCPT TO"
#define TXT_DATA "DATA"
#define TXT_RSET "RSET"
#define TXT_QUIT "QUIT"
#define TXT_NOOP "NOOP"

int main(int num, char *opts[]);
void addconfigdefaults(ConfigCraft &ar);
int runner(int num, ConfigCraft &ar, Acceptor &ac);
void help(void);
void resetinprocess(ConfigCraft &ar, NoteCraft &ls);
int gungrablock(ConfigCraft &ar, NoteCraft &ls, char *fn);

/* So we can make a list of these */
class destpack : public entry
  {
    public:
      destpack(void) : address("")
        {
          islocal = FALSE;
        }
      ~destpack(void){}
      emailaddr *getaddress(void) { return &address; }
      int getislocal(void) { return islocal; }
      char *getdefstore(void) { return defstore; }
      int getuserid(void) { return userid; }
      char *getrelayhost(void) { return relayhost; }
      void setaddr(emailaddr &a, int local, char *store, int u, char *relay)
        {
          address = a;
          islocal = local;
          defstore = store; // this is the defstore of the user
          userid = u;
          relayhost = relay;
        }
      void markend(FILE *f) // end of file mark
        {
          fprintf (f, ".\n");
        }
      int store(FILE *f)
        {
          int ret = fprintf (f, "%s\n", address.getfullpretty());
          if (ret < 0) return FALSE;
          ret = fprintf (f, "%d\n", islocal);
          if (ret < 0) return FALSE;
          ret = fprintf (f, "%s\n", (char *)defstore);
          if (ret < 0) return FALSE;
          ret = fprintf (f, "%d\n", userid);
          if (ret < 0) return FALSE;
          ret = fprintf (f, "%s\n", (char *)relayhost);
          if (ret < 0) return FALSE;
          return TRUE;
        }
      int restore(FILE *f)
        {
          Memory buf;
          if (readline(buf, f) == 0)
            return RET_ERROR;
          if (strcmp(buf, ".") == 0) // eof mark
            return RET_ENDOFDATA; // done
          emailaddr ad(buf);
          address = ad;

          if (readline(buf, f) == 0)
            return RET_ERROR;
          if ((buf[0] != '0') && (buf[0] != '1'))
            return RET_ERROR;
          islocal = val(buf);  

          if (readline(buf, f) == 0)
            return RET_ERROR;
          defstore = buf;

          if (readline(buf, f) == 0)
            return RET_ERROR;
          userid = val(buf);  

          if (readline(buf, f) == 0)
            return RET_ERROR;
          relayhost = buf;
          
          return RET_OK;
        }
    private:
      emailaddr address;
      int islocal;
      Memory defstore;
      int userid;
      Memory relayhost;
  };

class smtp
  {
    public:
      
      typedef enum 
        {
          needhelo = 0,
          needmailfrom = 1,
          needrcpt = 2, // need at least 1 RCPT
          needdata = 3  // expect data or another RCPT
        } statetype;
        
      smtp (ConfigCraft &Ar, Acceptor &Ac, NoteCraft &Ls, int num);
      ~smtp(void);
      int run(void);
      void delivercmdlinemail(void);
      void runqueue(void);

    private:
      void makequeuefilenames(Memory &aftemp, Memory &af, Memory &bf, Memory &idnoise);
      void maketempnamefromaf(Memory &newfn, char *origfn);
      int checkdomain(char *domain);

      int handlenewconfig(int num); // starting position
      void checkrereadconfig(void);

      void startsession(void);
      void heading(void);
      void response(char *s);
      char *expect(void);
      void processmailfrom(Memory &l);
      void processrcpt(Memory &l);
      void processdata(void);
      void reset(void);
    
      int makemessage(char *outfile, char *msgtext, char *attachment = NULL);
      void writeaffile(FILE *ff, char *bfname, char *mailfrom, char *storeid,
                       int attempts, int isbounce, int delivernow);
      int readaffile(FILE *ff, Memory &bfname, Memory &mailfrom, Memory &storeid,
                     int &attempts, int &isbounce, time_t &tm);
      void writeaferror(FILE *f, char *errstr);
      void readaferror(FILE *f, Memory &errstr);
      int getcurrentmessagedefstore(Memory &defstore);
      int deliverlocal(destpack *dp, char *bf, Memory &errstr);
      void insertoverquotamessage(tuser &user);
      int generateoverquotamessage(destpack *dp, char *bf);
      int generateundeliverablemessage(char *failedreciplist, char *bf, char *lasterror);
      int generatemessage(char *bf, Memory &subject, Memory &msg);
      int makemessage(char *outfile, char *sendto, char *subject, 
                            char *msgtext, char *attachment);
      void deadqueue(char *fn);
      int grablock(Memory &newname, char *oldname);
      int ungrablock(char *fn);
      void runitem(char *origfn);
      void dobadrcpt(void);

      ConfigCraft &ar;
      Acceptor &ac;
      NoteCraft &ls;
      dbfinder *finder;
      tdirectory *dirholder;
      
      socketio *sco;
      int state;
      Memory xmsg; // temporary persistent message returner
      Memory xhostname; // so we only have to get it once.

      zlist cachedomainlist; // cached accept-local-domain list
      Memory cachemsgdefstore; // cached message store defstore value
      time_t cachedomainreadtime;
      time_t cachemsgdefstorereadtime;
      zlist allowrelayinterfacelist;
      zlist allowrelayhostlist;

      /* Session state information, recip list and all that. */
      Memory peerclaim;
      Memory envelopemailfrom;
      Memory msgstoreid; // the md5hash to uid this message by in the msgstore
      zlist reciplist;
      int allowrelay; // can this client send mail out
      int sleeptime; // how long to wait between each bad rcpt

  };



#endif







