/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <Q/memm.h>

#include "userconfig.h"

/* The user configuration cgi program. All it does it call admin.
   All the functionality is there. Need a small script to set the
   current directory where the o3.cfg file is and the full path
   of the admin program you want to run. */
   
// you get here from a hit on the static page.

char *postbuf = NULL;

int main(int num, char *opts[])
  {

    // we never deviate from this...   
    printf("Content-Type: text/html\n\n");
    printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML //EN\">\n");

    printf("<html><body><center><font size=+2><B>O3 Mail System User Configuration</B></font>\n");
    printf("</center><br><br>\n");

    if (num < 2) // must pass name of admin program
      {
        printf ("Missing admin command to userconfig cgi.<br>\n");
        return ender();
      }

    process_getpost();

    char butn[200];
    Memory username;
    Memory domain;
    Memory password;
    Memory newpw1;
    Memory newpw2;
    Memory status;
    
    // get all the settings off the page hit

    if (get_var("username", butn, sizeof(butn)) != FALSE)
      username = butn; 
    if (get_var("domain", butn, sizeof(butn)) != FALSE)
      domain = butn; 
    if (get_var("password", butn, sizeof(butn)) != FALSE)
      password = butn;
    if (get_var("enableaccount", butn, sizeof(butn)) != FALSE)
      status = butn;
    if (get_var("newpassword", butn, sizeof(butn)) != FALSE)
      newpw1 = butn;
    if (get_var("newpassword2", butn, sizeof(butn)) != FALSE)
      newpw2 = butn;

    if ((strlen(username) == 0) || (strlen(domain) == 0) || (strlen(password) == 0)) 
      {
        printf ("You must enter a username, domain and password.");
        return ender();
      }

    // see which button they hit
    get_var("button", butn, sizeof(butn)); 
    
    if (strcmp(butn, "Change Password") == 0)
      {
        // check to see if they typed it right
        if (strcmp(newpw1, newpw2) != 0)
          {
            printf ("New password entries do not match.");
            return ender();
          }
        if ((strlen(newpw1) == 0) || (strlen(newpw2) == 0))
          {
            printf ("You must enter a new password.");
            return ender();
          }
        // make up admin string
        Memory cmd = opts[1];
        cmd.addstr(" chpassword ");
        cmd.addstr(username);
        cmd.addstr(" ");
        cmd.addstr(domain);
        cmd.addstr(" ");
        cmd.addstr(password);
        cmd.addstr(" ");
        cmd.addstr(newpw1);
        cmd.addstr(" > /tmp/adminoutput");
        int ret = system((char *)cmd);
        if (ret == -1)
          {
            printf("Unable to fork admin program %s. Error: %d %s", opts[1], errno, strerror(errno));
            return ender();
          }
          
        return outender(); // print out the response from admin program
      }
    else
    if (strcmp(butn, "Update Status") == 0)
      {
        // make up admin string
        Memory cmd = opts[1];
        cmd.addstr(" chstatus ");
        cmd.addstr(username);
        cmd.addstr(" ");
        cmd.addstr(domain);
        cmd.addstr(" ");
        cmd.addstr(password);
        cmd.addstr(" ");
        cmd.addstr(status); // should be either D or E
        cmd.addstr(" > /tmp/adminoutput");
        int ret = system((char *)cmd);
        if (ret == -1)
          {
            printf("Unable to fork admin program %s. Error: %d %s", opts[1], errno, strerror(errno));
            return ender();
          }
          
        return outender(); // print out the response from admin program
      }
    else
      printf ("Invalid request from userconfig.html\n");
    return ender();
  }

int outender(void)
  { // print out the response from admin program
    // and delete /tmp/adminoutput
    FILE *f = fopen("/tmp/adminoutput", "r");
    if (f == NULL)
      {  
        printf ("Unable to read admin response. Error %d %s\n", errno, strerror(errno));
        return ender();
      }
    Memory l;
    int ret = readline (l, f); // read in the header
    ret = readline(l, f);
    if (strlen(l) == 0)
      {
        printf ("Unable to read admin response.\n");
      }
    else
      printf ("%s", (char *)l);
    
    fclose (f);
    return ender();
  }

int ender(void)
  {
    printf("</center></body></html>\n");
    return 0;
  }  

char *get_query(char *src)
  {
    /* We own the memory */
    static char *ret = NULL;
    char *qq;
    char *str;
    char *fnd;
    char *start;
    char *end;
    
    /* now look for my piece */
    qq = (char *)getmem (strlen(postbuf) + 5);
    strcpy (qq, "&");
    strcat (qq, postbuf);
    
    str = (char *)getmem(strlen(src) + 5);
    strcpy (str, "&");
    strcat (str, src);
    strcat (str, "="); /* What to look for */
    
    fnd = strstr(qq, str); /* case sensitive */
    if (fnd == NULL)
      {
        freemem(qq);
        freemem(str);
        return NULL;
      }
    
    start = fnd+strlen(str);
    end = strchr (start, '&');
    if (end == NULL)
      end = start + strlen(start);
    
    if (ret != NULL)
      freemem(ret);
    ret = (char *)getmem(end-start+5);
    
    memcpy (ret, start, end-start);
    *(ret+(end-start)) = '\0';    
    
    freemem (str);
    freemem (qq);

    unescape (ret);
    if (strlen(ret) == 0)
      return NULL;
    return ret;
  }

void unescape(char *url)
  {
    register int i, j;

    for(i = 0, j = 0; url[j]; ++i, ++j) 
      {
        if((url[i] = url[j]) == '%') 
          {
            url[i] = x2c(&url[j + 1]);
            j += 2;
          }
        else 
          if (url[i] == '+')
            url[i] = ' ';
      }
    url[i] = '\0';  /* terminate it at the new length */
  }


int get_var (char *srch, char *buf, unsigned int maxlen)
  {
    /* This will get a variable returning only the length
       you ask for null terminated into your buffer and
       return TRUE or FALSE if not found */
    char *ret;
    
    maxlen--; /* Assume space for the \0 */
    
    ret = get_query (srch);
    if (ret == NULL) /* this will catch empties also */
      return FALSE;

    if (buf != NULL) /* Maybe they're just checking for existance */
      {
        if (maxlen <= strlen(ret))
          *(ret + maxlen) = '\0';
        strcpy (buf, ret);
      }
    return TRUE;
  }


void process_getpost(void)
  {
    /* This will read stdin if any and hold it in the post info area. */
    /* Then it will add QUERY STRING if any. */
    
    int varsize;
    char *buf;
    char *q;
    
    varsize = 100; /* jump by this much each read */
    
    if (postbuf != NULL)
      freemem(postbuf);
      
    postbuf = (char *)getmem(varsize+1);
    buf = postbuf;
    
    while (1)
      {
        int ret;

        ret = fread (buf, 1, varsize, stdin);
        if (ret < varsize)
          {
            buf += ret; /* Keep it up to speed for query string */
            *buf = '\0';
            break;
          }
        buf += varsize;
        varsize *= 2; /* bigger faster */
        reallocmem (postbuf, (buf-postbuf) + varsize + 1); /* Make room for null */
      }

    q = getenv("QUERY_STRING");
    if (q == NULL)
      return; 
    /* Otherwise add it */
    reallocmem (postbuf, (buf-postbuf) + 5 + strlen(q));
  
    if (strlen(postbuf) > 0)
      strcpy (buf, "&"); /* fake a new separator */
    strcat (buf, q);

  }

char x2c(char *x)
  {
    register char c;

    /* note: (x & 0xdf) makes x upper case */
    c  = (x[0] >= 'A' ? ((x[0] & 0xdf) - 'A') + 10 : (x[0] - '0'));
    c *= 16;
    c += (x[1] >= 'A' ? ((x[1] & 0xdf) - 'A') + 10 : (x[1] - '0'));
    return(c);
  }
