/*
    This file is part of O3.
    Copyright 2002, 2003 Stu Mark
    This program is distributed under the terms of the GNU 
    General Public License 

    O3 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    O3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with O3; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <sys/socket.h>
#include <sys/time.h>

#include <Q/memm.h>
#include <Q/zlist.h>
#include <Q/acceptor.h>
#include <Q/configcraft.h>
#include <Q/notecraft.h>
#include <flat/fc.h>

#include "global.h"
#include "io.h"
#include "dbfinder.h"
#include "utils.h"

extern "C" 
  { 
    #include <Q/md5.h> 
  }


/* system wide standalone utility functions. */

void fieldtomemory(Memory &dest, char *src, int fieldlen)
  {
    /* take the field pointed to by src and make dest out of it. */
    Memory al(fieldlen+1); // for nt
    *(al+fieldlen) = '\0'; // make sure it's a string
    memcpy((char *)al, src, fieldlen);
    trim(al);
    dest = al;
  }

void memorytofield(char *dest, char *src, int fieldlen, int lower)
  { // this is only for character/string data
    /* take the data pointed to by src and make field, space padded out of it. */

    int tocopy = strlen(src);
    if (tocopy > fieldlen)
      tocopy = fieldlen;
    memcpy (dest, src, tocopy);
    if (lower)
      {
        int lp;
        for (lp = 0; lp < tocopy; lp++)
          tolow(dest[lp]);
      }
    // pad the rest if any with spaces
    int rest = fieldlen - tocopy;
    if (rest > 0)
      memset(dest+tocopy, ' ', rest);
  }

void namefromaddress(Memory &name, char *n)
  {
    name = n;
    char *pos = strchr (name, '@');
    if (pos != NULL)
      *pos = '\0';
    trim(name);
  }
  
void domainfromaddress(Memory &domain, char *n)
  {
    domain = ""; // default
    char *pos = strchr(n, '@');
    if (pos != NULL)
      domain = pos+1;
    trim(domain);
  }



void setiplist (char *s, zlist &allowlist)
  {
    /* read this string and make a zlist of stringentry of ip addresses
       (basically an interface list)that we will allow relaying from */
    // accepts a space separated list of ip's or hostnames.
    char *pos = s;
    allowlist.WipeList();
    
    while (*pos != '\0')
      {
        while (*pos == ' ')
          pos++;
        Memory one;
        int ishostname = FALSE;
        while ((*pos != '\t') && (*pos != ' ') && (*pos != '\0'))
          {
            if (!(((*pos >= '0') && (*pos <= '9')) || (*pos == '.')))
              ishostname = TRUE; // any non .ip makes it a hostname
            one.addchar(*pos);
            pos++;
          }
        if (strlen(one) > 0)
          {
            if (ishostname)
              {
                unsigned long ret = lookuphost(one);
                stringfromip(ret, one);
              }
            if (strlen(one) > 0) // make sure it worked
              {
                stringentry *str = new stringentry(one);
                allowlist.Add(str);
              }
          }
      }
  }
  

int isinlist(zlist &list, char *address)
  {
    // return true if this address matches something
    // (partial or full) in the zlist of ip addresses
    // stole this from acceptor
    int lc = list.ListCount();
    if (lc == 0)
      return FALSE; // list is empty

    int lp;
    for (lp = 0; lp < lc; lp++)
      {
        stringentry *s = (stringentry *)list.GetItem(lp);
        Memory one = s->getstr();
        
        Memory comp(address);
        UINT ps = strlen(one);
        /* Make them the same size if we're comparing class C's or something
           192.168.1.2 becomes 192.168.1 then compare that.
           True, it will allow 192.168.10 also, but not a big deal.
           If you care that much, then put the trailing dot: 192.168.1. */
           
        if (strlen(comp) >= ps)
          *(comp+ps) = '\0'; // shorten it to match same size
        if (strcmp(comp, one) == 0)
          return TRUE;
      }
    return FALSE;
  }

void splithostportpath(char *pd, Memory &server, int &port, Memory &path)
  {
    /* Given a pd of "deadpelican.com:1049:/var/mail"
       split it into server and port and path */

    path = "/tmp/"; // in case it's bad
    server = pd;
    port = 0;
    char *x = strchr (server, ':');
    if (x == NULL)
      return;
    *x = '\0';  // make the server string
    x++; // skip :
    char *y = strchr (x, ':');
    if (y)
      {
        *y = '\0';
        y++;
        path = y;
      }
    port = val(x);
  }
  
int getpeeripstring(Memory &ip, Acceptor &ac)
  {
    /* Return TRUE or FALSE, result goes in ip */
    struct sockaddr_in *sin;
    int namelen;
    
    /* Get the address this connection came in on */
    Memory inaddr(50); // bigger than a 4 dot address
    ac.getpeeraddress(sin, namelen);
    unsigned int i = sin->sin_addr.s_addr; // for some reason this doesn't need flip on either endian
    unsigned char *x = (unsigned char *)&i;
    // absolutely shameful
    UINT x0 = (UINT)x[0];
    UINT x1 = (UINT)x[1];
    UINT x2 = (UINT)x[2];
    UINT x3 = (UINT)x[3];
    sprintf (inaddr, "%u.%u.%u.%u", x0, x1, x2, x3);
    
    ip = inaddr;
    return TRUE;
  }

int getfromorreplyto(Memory &from, Memory &envelopemailfrom, char *bf)
  {
    /* Run through the headers in the mail file bf
       and get the from or reply-to header, and return
       it in from */
    // there's a lot of room for error here, and no logging... 
    FILE *f = fopen(bf, TXT_FOPENREAD);
    if (f == NULL)
      return RET_ERROR;
    filecloser x(f);
    
    from = "";
    while (1)
      {
        Memory buf;
        int ret = readline(buf, f);
        if (ret == 0)
          break; // end of file
        if (strlen(buf) == 0)
          break; // end of headers
        
        /* Only get from if empty, get reply-to always, this has
           the effect of replyto overriding from */
        if (strncasecmp(TXT_FROM, buf, strlen(TXT_FROM)) == 0)
          if (strlen(from) == 0)
            from = buf + strlen(TXT_FROM);
        
        if (strncasecmp(TXT_REPLY_TO, buf, strlen(TXT_REPLY_TO)) == 0)
          from = buf + strlen(TXT_REPLY_TO);
      }
    trim(from);
    if (strlen(from) == 0)
      from = envelopemailfrom; // last resort, try this.

    trim(from);
    if (strlen(from) > 0)
      return RET_OK; // got something
    return RET_NOTFOUND; // no envelope or info in the header
  }

void md5hash (Memory &output, char *hashme)
  {
    /* return a 32 char string of hex of hash */
    char md5hash[33];
    unsigned char buf[16];
    int lp;
    
    md5hash[0] = '\0';
    md5_calc(buf, (unsigned char *)hashme, strlen(hashme));
    for (lp = 0; lp < 16; lp++)
      sprintf(md5hash + lp*2, "%02X", (int)buf[lp]);
    output = md5hash;
  }

char *txt_day[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
char *txt_mon[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

void generaterfc822date(Memory &dt)
  {
    // make an RFC 822 looking date.

    // get the local tz string from the system
    struct timeval tp;
    struct timezone tzp;
    gettimeofday(&tp, &tzp);
    Memory tzsign;
    if (tzp.tz_minuteswest > 0)
      tzsign = "-";
    else
      {
        tzsign = "+";
        tzp.tz_minuteswest = -tzp.tz_minuteswest; // abs(x)
      }
    int tzhour = tzp.tz_minuteswest / 60;
    int tzminute = tzp.tz_minuteswest % 60;
    Memory tzone(10);
    sprintf (tzone, "%c%02d%02d", *tzsign, tzhour, tzminute);
    
    Memory d(80);
    time_t t;
    struct tm dat;
    t = time(NULL);
    memcpy (&dat, localtime(&t), sizeof(dat));
    sprintf (d, "%s, %02d %s %d %02d:%02d:%02d %s", txt_day[dat.tm_wday],
             dat.tm_mday, txt_mon[dat.tm_mon], dat.tm_year+1900, 
             dat.tm_hour, dat.tm_min, dat.tm_sec, (char *)tzone);
    dt = d;
  }
  
void generateboundary(Memory &boundary, char *starter)
  {
    /* Make a relativly unique boundary, based on hash of
       time and starter */
       
    boundary = "part-"; // they'll never understand

    Memory hash;
    time_t t;

    Memory uid(50 + strlen(starter));
    strcpy (uid, starter);
    t = time(NULL);
    strcpy (uid + strlen(uid), ctime(&t));

    md5hash(hash, uid);
    boundary.addstr(hash);
    boundary.addstr("-part");
    /* they just can't understand, that's right, so We Out. */
  }

void foldline(FILE *f, char *s2, int indent)
  {
    // fold s into file f and CRLF at the end
    // xxxz make this smarter to notice CRLFs embedded in the line to be output
    Memory slock(s2); // because we're going to butcher it.
    char *s = slock;
    trim(s);
    while (strlen(s) > RFCLINESIZE)
      {
        char *p = s + RFCLINESIZE;
        while (p > s)
          { 
            if (*p == ' ')
              break;
            p--;
          }
        if (p == s) // hmm, no spaces
          {
            fprintf (f, "%s%s", (char *)s, CRLF);
            break; // no spaces, no folding
          }
        *p = '\0';
        fprintf (f, "%s%s", (char *)s, CRLF);
        if (indent)
          fprintf (f, "    "); // indent next line
        s = p + 1; // skip \0. 
      }
    fprintf (f, "%s%s", s, CRLF);
  }

void formatline(FILE *f, char *s2)
  {
    foldline(f, s2, FALSE); // fold with no indent
  }

int recalculatequota(FlatConnection *dbc, UINT userid, ConfigCraft &ar, NoteCraft &ls)
  {
    /* Select all the messages from this user's folders
       sum up the sizes and update the user record
       with the current use. FALSE if can't do it. */
    /* This isn't the cheapest thing in the world so don't do it too often */

    tuser user(*dbc, ar, ls);
    int ret = user.lookup(userid);
    if (ret != RET_OK)
      {
        ls(NOTEERR) << "Sanity failure. Unable to find user in mailstore for quota recalc." << endl;
        return FALSE;
      }

    Memory key(sizeof(userid));
    memcpy (key, &userid, sizeof(userid));

    tmsgptr fmsgptr(*dbc, ar, ls);

    Memory resp;
    zlist list;
    fresp fret = fmsgptr.iselect(key, list, sizeof(userid));
    if (fret != FR_OK)
      {
        ls(NOTEERR) << "Select failed on message list lookup for quota recalc." << endl;
        return FALSE;
      }
    
    int mx = list.ListCount();
    int lp;
    UINT q = 0;
    for (lp = 0; lp < mx; lp++)
      {
        stringentry *s = (stringentry *)list.GetItem(lp);
        msgptrrec *r = (msgptrrec *)(char *)s->getstr();
        q += r->msgsize;
      }
    user.getdata().quotaused = q;
    int dret = user.update(userid);
    if (dret != RET_OK)
      {
        ls(NOTEERR) << "Unable to update user quota after quota recalc." << endl;
        return FALSE;
      }
    return TRUE;
  }

void md5_calc(unsigned char *outbuf, unsigned char *hashme, int hashlen)
  {
    MD5_CTX c;
  
    MD5Init (&c);
    MD5Update(&c, hashme, hashlen);
    MD5Final (outbuf, &c);
  }
  
void fixsigpipe(NoteCraft &ls)
  {
    // Turn SIGPIPE off
    struct sigaction Action;
    Action.sa_handler = SIG_IGN; // let signal fall through to errno

    sigset_t s_t;
    sigemptyset(&s_t); 

    Action.sa_mask = s_t; // no other signals needed
    Action.sa_flags = 0; // no special flags needed

    int ret = sigaction(SIGPIPE, &Action, NULL); /* Don't need to save old handler */
    if (ret != 0)
      {
        ls(NOTESEV) << "Unable to set signal handler for sigpipe." << endl
           << endl << strerror(errno) << endl;
      }
  }
  
int checkfinder(ConfigCraft &ar, NoteCraft &ls, dbfinder *&finder, tdirectory *&dirholder)
  { 
    /* We either set or remove finder denoting the state of our connection
       to the database. */
    if (finder != NULL)
      return TRUE;
    if (dirholder != NULL)
      delete dirholder;
    dirholder = NULL;
    finder = new dbfinder(ar, ls, ar[TXT_PRIME_DEFSTORE]);

    if (finder->getprimeconn() != NULL)
      {
        dirholder = new tdirectory(*finder, ar, ls); // this is a connection pooling optomization
        dirholder->checkversion();
        return TRUE;
      }
    ls(NOTESEV) << "Unable to connect to prime defstore: " << ar[TXT_PRIME_DEFSTORE] << endl;
    dirholder = NULL;
    delete finder;
    finder = NULL;
    return FALSE;
  }

int iswhitespace(char p)
  {
    if ((p == ' ') || (p == '\t'))
      return TRUE;
    return FALSE;
  }
  
int gettoken(char *&p, Memory &v)
  {
    /* Given a pointer, find the next non whitespace
       add pieces to v until whitespace.
       return FALSE if there's nothing there. */
    v = Memory(50);
    if (p == NULL)  
      return FALSE;
    while (iswhitespace(*p))
      p++;
    while ((*p != '\0') && (iswhitespace(*p) == FALSE))
      { 
        v.addchar(*p);
        p++;
      }
    if (strlen(v) == 0)
      {
        p = NULL;
        return FALSE;
      }
    trim(v);
    return TRUE;
  }

